# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.fsm import StreamFSM
from noc.lib.ecma48 import strip_control_sequences
from noc.lib.registry import Registry
from noc.lib.nbsocket import PTYSocket
from noc.lib.debug import format_frames,get_traceback_frames
from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP
from noc.sa.profiles import profile_registry
import logging,re,threading,Queue,urllib,httplib,random,base64,hashlib,cPickle,sys,time

try:
    from pysnmp.carrier.asynsock.dispatch import AsynsockDispatcher
    from pysnmp.carrier.asynsock.dgram import udp
    from pyasn1.codec.ber import encoder, decoder
    from pysnmp.proto import api
    HAS_SNMP=True
except ImportError:
    HAS_SNMP=False


##
##
##
scheme_id={
    "telnet" : TELNET,
    "ssh"    : SSH,
    "http"   : HTTP,
}

##
##
##
class ScriptProxy(object):
    def __init__(self,parent):
        self._parent=parent
    def __getattr__(self,name):
        return ScriptCallProxy(self._parent,self._parent.profile.scripts[name])

class ScriptCallProxy(object):
    def __init__(self,parent,script):
        self.parent=parent
        self.script=script
        
    def __call__(self,**kwargs):
        s=self.script(self.parent.profile,self.parent.activator,self.parent.access_profile,parent=self.parent,**kwargs)
        return s.guarded_run()
##
##
##
class ScriptRegistry(Registry):
    name="ScriptRegistry"
    subdir="profiles"
    classname="Script"
    apps=["noc.sa"]
    def register_generics(self):
        for c in [c for c in self.classes.values() if c.name and c.name.startswith("Generic.") and c.requires]:
            g,name=c.name.split(".")
            for p in profile_registry.classes:
                s_name=p+"."+name
                # Do not register generic when specific exists
                if s_name in self.classes:
                    continue
                to_register=True
                for r_name,r_interface in c.requires:
                    rs_name=p+"."+r_name
                    if rs_name not in self.classes or not self.classes[rs_name].implements_interface(r_interface):
                        to_register=False
                        break
                if to_register:
                    logging.debug("Script Registry: Register generic %s"%s_name)
                    self.classes[s_name]=c
                    profile_registry[p].scripts[name]=c
    def register_all(self):
        super(ScriptRegistry,self).register_all()
        self.register_generics()

script_registry=ScriptRegistry()
##
##
##
class ScriptBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        m.implements=[c() for c in m.implements]
        script_registry.register(m.name,m)
        if m.name and not m.name.startswith("Generic."):
            pv,pos,sn=m.name.split(".",2)
            profile_registry["%s.%s"%(pv,pos)].scripts[sn]=m
        return m
##
##
##
class Script(threading.Thread):
    __metaclass__=ScriptBase
    name=None
    description=None
    # Interfaces list. Each element of list must be Interface subclass
    implements=[]
    # Scripts required by generic script.
    # For common scripts - empty list
    # For generics - list of pairs (script_name,interface)
    requires=[]
    # Constants
    TELNET=scheme_id["telnet"]
    SSH=scheme_id["ssh"]
    HTTP=scheme_id["http"]

    def __init__(self,profile,activator,access_profile,parent=None,**kwargs):
        self.parent=parent
        self.access_profile=access_profile
        if self.access_profile.address:
            p=self.access_profile.address
        elif self.access_profile.path:
            p=self.access_profile.path
        else:
            p="<unknown>"
        self.debug_name="script-%s-%s"%(p,self.name)
        super(Script,self).__init__(name=self.debug_name,kwargs=kwargs)
        self.activator=activator
        self.profile=profile
        self.cli_provider=None
        self.http=HTTPProvider(self.access_profile)
        if HAS_SNMP:
            self.snmp=SNMPProvider(self.access_profile)
        else:
            self.snmp=None
        self.status=False
        self.result=None
        self.error_traceback=None
        self.strip_echo=True
        self.kwargs=kwargs
        self.scripts=ScriptProxy(self)
    
    @classmethod
    def implements_interface(cls,interface):
        for i in cls.implements:
            if type(i)==interface:
                return True
        return False
        
    def debug(self,msg):
        logging.debug("[%s] %s"%(self.debug_name,msg))
        
    def guarded_run(self):
        self.debug("Guarded run")
        # Enforce interface type checking
        for i in self.implements:
            self.kwargs=i.clean(**self.kwargs)
        # Calling script body
        result=self.execute(**self.kwargs)
        # Enforce interface result checking
        for i in self.implements:
            result=i.clean_result(result)
        self.debug("Script returns with result: %s"%result)
        return result
        
    def serialize_result(self,result):
        return cPickle.dumps(result)
        
    def run(self):
        self.debug("Running")
        try:
            self.result=self.serialize_result(self.guarded_run())
        except:
            t,v,tb=sys.exc_info()
            r=[str(t),str(v)]
            r+=[format_frames(get_traceback_frames(tb))]
            self.error_traceback="\n".join(r)
            self.debug("Script traceback:\n%s"%self.error_traceback)
        self.debug("Closing")
        if self.cli_provider:
            self.activator.request_call(self.cli_provider.close)
        self.activator.on_script_exit(self)
        
    def execute(self,**kwargs):
        return None
    
    def request_cli_provider(self):
        if self.parent:
            self.cli_provider=self.parent.request_cli_provider()
        elif self.cli_provider is None:
            self.debug("Running new provider")
            if self.access_profile.scheme==TELNET:
                s_class=CLITelnetSocket
            elif self.access_profile.scheme==SSH:
                s_class=CLISSHSocket
            else:
                raise Exception("Invalid access scheme '%d' for CLI"%self.access_profile.scheme)
            self.cli_provider=s_class(self.activator.factory,self.profile,self.access_profile)
            self.cli_provider.queue.get(block=True) # Wait until provider in PROMPT
            self.debug("CLI Provider is ready")
        return self.cli_provider
        
    def cli(self,cmd):
        self.debug("cli(%s)"%cmd)
        self.request_cli_provider()
        self.cli_provider.submit(cmd)
        data=self.cli_provider.queue.get(block=True)
        if self.strip_echo and data.lstrip().startswith(cmd):
            data=self.strip_first_lines(data.lstrip())
        self.debug("cli() returns:\n---------\n%s\n---------"%data)
        return data
    ##
    ## Clean up config from all unnecessary trash
    ##
    def cleaned_config(self,config):
        return self.profile.cleaned_config(config)
    ##
    ##
    ##
    def strip_first_lines(self,text,lines=1):
        t=text.split("\n")
        if len(t)<=lines:
            return ""
        else:
            return "\n".join(t[lines:])
    ##
    ## Expands expressions like "1,2,5-7" to [1,2,5,6,7]
    ##
    def expand_rangelist(self,s):
        result={}
        for x in s.split(","):
            x=x.strip()
            if x=="":
                continue
            if "-" in x:
                l,r=[int(y) for y in x.split("-")]
                if l>r:
                    x=r
                    r=l
                    l=x
                for i in range(l,r+1):
                    result[i]=None
            else:
                result[int(x)]=None
        return sorted(result.keys())
##
##
##
class TimeOutError(Exception): pass
##
##
##
class CLI(StreamFSM):
    FSM_NAME="CLI"
    DEFAULT_STATE="START"
    STATES={
        "START":{
            "USERNAME"            : "USERNAME",
            "PASSWORD"            : "PASSWORD",
            "UNPRIVELEGED_PROMPT" : "UNPRIVELEGED_PROMPT",
            "PROMPT"              : "PROMPT"
        },
        "USERNAME":{
            "USERNAME"            : "FAILURE",
            "PASSWORD"            : "PASSWORD",
            "UNPRIVELEGED_PROMPT" : "UNPRIVELEGED_PROMPT",
            "PROMPT"              : "PROMPT"
        },
        "PASSWORD":{
            "USERNAME"            : "FAILURE",
            "PASSWORD"            : "FAILURE",
            "UNPRIVELEGED_PROMPT" : "UNPRIVELEGED_PROMPT",
            "PROMPT"              : "PROMPT"
        },
        "SUPER_PASSWORD" : {
            "UNPRIVELEGED_PROMPT" : "FAILURE",
            "PASSWORD"            : "FAILURE",
            "PROMPT"              : "PROMPT",
        },
        "UNPRIVELEGED_PROMPT":{
            "PASSWORD"            : "SUPER_PASSWORD",
            "PROMPT"              : "PROMPT",
        },
        "PROMPT":{
            "PROMPT"      : "PROMPT",
            "PAGER"       : "PROMPT",
            "PAGER_START" : "PROMPT",
            "PAGER_END"   : "PROMPT",
            "CLOSE"       : "CLOSED",
        },
        "FAILURE":{
        },
        "CLOSED":{
        }
    }
    
    def __init__(self,profile,access_profile):
        self.profile=profile
        self.access_profile=access_profile
        self.queue=Queue.Queue()
        self.is_ready=False
        self.collected_data=""
        StreamFSM.__init__(self)
    
    def on_read(self,data):
        self.debug("on_read: %s"%repr(data))
        if self.profile.rogue_chars:
            for rc in self.profile.rogue_chars:
                data=data.replace(rc,"")
        self.feed(data,cleanup=strip_control_sequences)
    
    def submit(self,msg):
        self.debug("submit(%s)"%repr(msg))
        self.write(msg+self.profile.command_submit)
    
    def on_START_enter(self):
        p=[
            (self.profile.pattern_username,"USERNAME"),
            (self.profile.pattern_password,"PASSWORD"),
        ]
        if self.profile.pattern_unpriveleged_prompt:
            p+=[
                (self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT"),
            ]
        p+=[
            (self.profile.pattern_prompt,"PROMPT"),
        ]
        self.set_patterns(p)
    
    def on_USERNAME_enter(self):
        self.set_patterns([
            (self.profile.pattern_password, "PASSWORD"),
            (self.profile.pattern_prompt,   "PROMPT"),
        ])
        self.submit(self.access_profile.user)
        
    def on_PASSWORD_enter(self):
        p=[(self.profile.pattern_prompt, "PROMPT")]
        if self.profile.pattern_unpriveleged_prompt:
            p+=[
                (self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT"),
            ]
        p+=[
            (self.profile.pattern_username, "USERNAME"),
            (self.profile.pattern_password, "PASSWORD")
            ]
        self.set_patterns(p)
        self.submit(self.access_profile.password)
        
    def on_UNPRIVELEGED_PROMPT_enter(self):
        self.set_patterns([
            (self.profile.pattern_prompt,   "PROMPT"),
            (self.profile.pattern_password, "PASSWORD"),
        ])
        self.submit(self.profile.command_super)
    
    def on_SUPER_PASSWORD_enter(self):
        self.set_patterns([
            (self.profile.pattern_prompt, "PROMPT"),
            (self.profile.pattern_password, "PASSWORD")
        ])
        sp=self.access_profile.super_password
        if not sp:
            sp=""
        self.submit(sp)
        
    def on_PROMPT_enter(self):
        if not self.is_ready:
            self.queue.put(None) # Signal provider passing into PROMPT state
            self.is_ready=True
        p=[
            (self.profile.pattern_prompt, "PROMPT"),
            (self.profile.pattern_more,   "PAGER"),
            ]
        if self.profile.pattern_more_start:
            p+=[(self.profile.pattern_more_start, "PAGER_START")]
        if self.profile.pattern_more_end:
            p+=[(self.profile.pattern_more_end, "PAGER_END")]
        self.set_patterns(p)
        
    def on_PROMPT_match(self,data,match):
        if match.re.pattern==self.profile.pattern_more:
            self.collected_data+=data
        elif match.re.pattern==self.profile.pattern_prompt:
            self.queue.put(self.collected_data+data)
            self.collected_data=""
        
    def on_PROMPT_PAGER(self):
        self.write(self.profile.command_more)
    
    def on_PROMPT_PAGER_START(self):
        if self.profile.command_more_start is None:
            self.write(self.profile.command_more)
        else:
            self.write(self.profile.command_more_start)
    
    def on_PROMPT_PAGER_END(self):
        if self.profile.command_more_end is None:
            self.write(self.profile.command_more)
        else:
            self.write(self.profile.command_more_end)

    def on_FAILURE_enter(self):
        self.set_patterns([])

##
##
##
class ScriptSocket(object):pass
#
##
##
class CLITelnetSocket(ScriptSocket,CLI,PTYSocket):
    def __init__(self,factory,profile,access_profile):
        logging.debug("CLITelnetSocket connecting '%s'"%access_profile.address)
        CLI.__init__(self,profile,access_profile)
        PTYSocket.__init__(self,factory,["/usr/bin/telnet",access_profile.address])
        ScriptSocket.__init__(self)
##
##
##
class CLISSHSocket(ScriptSocket,CLI,PTYSocket):
    def __init__(self,factory,profile,access_profile):
        logging.debug("CLISSHSocket connecting '%s'"%access_profile.address)
        CLI.__init__(self,profile,access_profile)
        PTYSocket.__init__(self,factory,["/usr/bin/ssh","-o","StrictHostKeyChecking no","-l",access_profile.user,access_profile.address])
        ScriptSocket.__init__(self)
##
##
##
class HTTPProvider(object):
    def __init__(self,access_profile):
        self.access_profile=access_profile
        self.authorization=None
    
    def request(self,method,path,params=None,headers={}):
        if self.authorization:
            headers["Authorization"]=self.authorization
        conn=httplib.HTTPConnection(self.access_profile.address)
        conn.request(method,path,params,headers)
        response=conn.getresponse()
        try:
            if response.status==200:
                return response.read()
            elif response.status==401 and self.authorization is None:
                self.set_authorization(response.getheader("www-authenticate"),method,path)
                return self.request(method,path,params,headers)
            else:
                raise Exception("HTTP Error: %s"%response.status)
        finally:
            conn.close()
            
    def set_authorization(self,auth,method,path):
        scheme,data=auth.split(" ",1)
        scheme=scheme.lower()
        d={}
        for s in data.split(","):
            s=s.strip()
            if "=" in s:
                k,v=s.split("=",1)
                if v.startswith("\"") and v.endswith("\""):
                    v=v[1:-1]
                d[k]=v
            else:
                d[s]=None
        if scheme=="basic":
            self.authorization="Basic %s"%base64.b64encode("%s:%s"%(self.access_profile.user,self.access_profile.password)).strip()
        elif scheme=="digest":
            H = lambda x: hashlib.md5(x).hexdigest()
            KD= lambda x,y: H("%s:%s"%(x,y))
            A1="%s:%s:%s"%(self.access_profile.user,d["realm"],self.access_profile.password)
            A2="%s:%s"%(method,path)
            f={
                "username": self.access_profile.user,
                "realm"   : d["realm"],
                "nonce"   : d["nonce"],
                "uri"     : path,
            }
            if "qop" not in d:
                noncebit="%s:%s"%(d["nonce"],H(A2))
            elif d["qop"]=="auth":
                nc="00000001"
                cnonce=H(str(random.random()))
                f["nc"]=nc
                f["cnonce"]=cnonce
                f["qop"]=d["qop"]
                noncebit="%s:%s:%s:%s:%s"%(d["nonce"],nc,cnonce,d["qop"],H(A2))
            else:
                raise Exception("qop not supported: %s"%d["qop"])
            f["response"]=KD(H(A1),noncebit)
            self.authorization="Digest "+", ".join(["%s=\"%s\""%(k,v) for k,v in f.items()])
        else:
            raise Exception("Unknown auth method: %s"%scheme)
    
    def get(self,path,params=None,headers={}):
        return self.request("GET",path)
    
    def post(self,path,params=None,headers={}):
        if params:
            params=urllib.urlencode(params)
            headers["Content-Type"]="application/x-www-form-urlencoded"
        return self.request("POST",path,params,headers)
##
##
##
class SNMPProvider(object):
    TimeOutError=TimeOutError
    def __init__(self,access_profile):
        self.access_profile=access_profile
        self.queue=Queue.Queue()
    
    def str_to_oid(self,s):
        return tuple([int(x) for x in s.split(".")])
        
    def get(self,oid):
        self.run_get(oid)
        return self.queue.get(block=True)
    
    def run_get(self,oid):
        logging.debug("SNMP GET %s"%oid)
        # Protocol version to use
        protocol = api.protoModules[api.protoVersion1]
        # Build PDU
        req_pdu =  protocol.GetRequestPDU()
        protocol.apiPDU.setDefaults(req_pdu)
        protocol.apiPDU.setVarBinds(req_pdu, ((self.str_to_oid(oid), protocol.Null()),))
        # Build message
        req = protocol.Message()
        protocol.apiMessage.setDefaults(req)
        protocol.apiMessage.setCommunity(req, self.access_profile.snmp_ro)
        protocol.apiMessage.setPDU(req, req_pdu)
        def timer_callback(timeNow, start_time=time.time()):
            if timeNow - start_time > 3:
                raise TimeOutError
        def recv_callback(transportDispatcher, transportDomain, transportAddress, msg, req_pdu=req_pdu):
            while msg:
                rsp_msg, msg = decoder.decode(msg, asn1Spec=protocol.Message())
                rspPDU = protocol.apiMessage.getPDU(rsp_msg)
                # Match response to request
                if protocol.apiPDU.getRequestID(req_pdu)==protocol.apiPDU.getRequestID(rspPDU):
                    # Check for SNMP errors reported
                    errorStatus = protocol.apiPDU.getErrorStatus(rspPDU)
                    #if errorStatus:
                    #    print errorStatus.prettyPrint()
                    #else:
                    for oid, val in protocol.apiPDU.getVarBinds(rspPDU):
                        self.queue.put(str(val))
                    transportDispatcher.jobFinished(1)
            return msg
        transportDispatcher = AsynsockDispatcher()
        transportDispatcher.registerTransport(udp.domainName, udp.UdpSocketTransport().openClientMode())
        transportDispatcher.registerRecvCbFun(recv_callback)
        transportDispatcher.registerTimerCbFun(timer_callback)
        transportDispatcher.sendMessage(encoder.encode(req), udp.domainName, (self.access_profile.address, 161))
        transportDispatcher.jobStarted(1)
        transportDispatcher.runDispatcher()
        transportDispatcher.closeDispatcher()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from noc.lib.validators import check_asn,check_as_set,is_ipv4,is_cidr
from noc.lib.tt import tt_url,admin_tt_url
from noc.lib.rpsl import rpsl_format
from noc.sa.profiles import profile_registry
from noc.cm.models import PrefixList
from noc.sa.models import AdministrativeDomain
from noc.main.menu import Menu
import random,sets

class RIR(models.Model):
    class Meta:
        verbose_name="RIR"
        verbose_name_plural="RIRs"
        ordering=["name"]
    name=models.CharField("name",max_length=64,unique=True)
    whois=models.CharField("whois",max_length=64,blank=True,null=True)
    def __unicode__(self):
        return self.name

class Person(models.Model):
    class Meta:
        verbose_name="Person"
        verbose_name_plural="Persons"
    nic_hdl=models.CharField("nic-hdl",max_length=64,unique=True)
    person=models.CharField("person",max_length=128)
    address=models.TextField("address")
    phone=models.TextField("phone")
    fax_no=models.TextField("fax-no",blank=True,null=True)
    email=models.TextField("email")
    rir=models.ForeignKey(RIR,verbose_name="RIR")
    extra=models.TextField("extra",blank=True,null=True)
    def __unicode__(self):
        return "%s (%s)"%(self.nic_hdl,self.person)
    def _rpsl(self):
        s=[]
        s+=["person: %s"%self.person]
        s+=["nic-hdl: %s"%self.nic_hdl]
        s+=["address: %s"%x for x in self.address.split("\n")]
        s+=["phone: %s"%x for x in self.phone.split("\n")]
        if self.fax_no:
            s+=["fax-no: %s"%x for x in self.fax_no.split("\n")]
        s+=["email: %s"%x for x in self.email.split("\n")]
        if self.extra:
            s+=[self.extra]
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    def rpsl_link(self):
        return "<A HREF='/peer/person/%d/rpsl/'>RPSL</A>"%self.id
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True

class Maintainer(models.Model):
    class Meta:
        verbose_name="Maintainer"
        verbose_name_plural="Maintainers"
    maintainer=models.CharField("mntner",max_length=64,unique=True)
    description=models.CharField("description",max_length=64)
    auth=models.TextField("auth")
    rir=models.ForeignKey(RIR,verbose_name="RIR")
    admins=models.ManyToManyField(Person,verbose_name="admin-c")
    extra=models.TextField("extra",blank=True,null=True)
    def __unicode__(self):
        return self.maintainer
    def _rpsl(self):
        s=[]
        s+=["mntner: %s"%self.maintainer]
        s+=["descr: %s"%self.description]
        s+=["auth: %s"%x for x in self.auth.split("\n")]
        s+=["admins: %s"%x.nic_hdl for x in self.admins.all()]
        s+=["mnt-by: %s"%self.maintainer]
        if self.extra:
            s+=[self.extra]
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    def rpsl_link(self):
        return "<A HREF='/peer/maintainer/%d/rpsl/'>RPSL</A>"%self.id
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True

class AS(models.Model):
    class Meta:
        verbose_name="AS"
        verbose_name_plural="ASes"
    maintainer=models.ForeignKey(Maintainer,verbose_name="Maintainer")
    asn=models.IntegerField("ASN",unique=True) # ,validator_list=[check_asn]
    description=models.CharField("Description",max_length=64)
    rpsl_header=models.TextField("RPSL Header",null=True,blank=True)
    rpsl_footer=models.TextField("RPSL Footer",null=True,blank=True)
    def __str__(self):
        return "AS%d (%s)"%(self.asn,self.description)
    def __unicode__(self):
        return u"AS%d (%s)"%(self.asn,self.description)
    @classmethod
    def default_as(cls):
        return AS.objects.get(asn=0)
    def _rpsl(self):
        sep="remark: %s"%("-"*72)
        s=[]
        if self.rpsl_header:
            s+=self.rpsl_header.split("\n")
        s+=["aut-num: AS%s"%self.asn]
        groups={}
        peers={}
        for peer in self.peer_set.all():
            groups[peer.peer_group.id]=None
        for pg in PeerGroup.objects.filter(id__in=groups.keys()):
            if pg.description:
                s+=[sep]
                s+=["remark: -- %s"%x for x in pg.description.split("\n")]
                s+=[sep]
            for peer in self.peer_set.filter(peer_group__exact=pg):
                rpsl=peer.rpsl
                if rpsl in peers:
                    continue
                peers[rpsl]=None
                s+=rpsl.split("\n")
        if self.rpsl_footer:
            s+=[sep]
            s+=self.rpsl_footer.split("\n")
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    
    def rpsl_link(self):
        return "<A HREF='/peer/AS/%d/rpsl/'>RPSL</A>"%self.asn
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True
    
    def _dot(self):
        s=["graph {"]
        all_peers=Peer.objects.filter(local_asn__exact=self)
        uplinks={}
        peers={}
        downlinks={}
        for p in all_peers:
            if p.import_filter=="ANY" and p.export_filter!="ANY":
                uplinks[p.remote_asn]=p
            elif p.export_filter=="ANY":
                downlinks[p.remote_asn]=p
            else:
                peers[p.remote_asn]=p
        asn="AS%d"%self.asn
        for subgraph,peers in [("uplinks",uplinks.values()),("peers",peers.values()),("downlinks",downlinks.values())]:
            s+=["subgraph %s {"%subgraph]
            for p in peers:
                attrs=["taillabel=\"%s\""%p.import_filter,"headlabel=\"%s\""%p.export_filter]
                if p.import_filter=="ANY":
                    attrs+=["arrowtail=open"]
                if p.export_filter=="ANY":
                    attrs+=["arrothead=open"]
                s+=["    %s -- AS%d [%s];"%(asn,p.remote_asn,",".join(attrs))]
            s+=["}"]
        s+=["}"]
        return "\n".join(s)
    dot=property(_dot)

class CommunityType(models.Model):
    class Meta:
        verbose_name="Community Type"
        verbose_name_plural="Community Types"
    name=models.CharField("Description",max_length=32,unique=True)
    def __unicode__(self):
        return self.name

class Community(models.Model):
    class Meta:
        verbose_name="Community"
        verbose_name_plural="Communities"
    community=models.CharField("Community",max_length=20,unique=True)
    type=models.ForeignKey(CommunityType,verbose_name="Type")
    description=models.CharField("Description",max_length=64)
    def __unicode__(self):
        return self.community

class ASSet(models.Model):
    class Meta:
        verbose_name="ASSet"
        verbose_name_plural="ASSets"
    name=models.CharField("Name",max_length=32,unique=True) # ,validator_list=[check_as_set]
    description=models.CharField("Description",max_length=64)
    members=models.TextField("Members",null=True,blank=True)
    rpsl_header=models.TextField("RPSL Header",null=True,blank=True)
    rpsl_footer=models.TextField("RPSL Footer",null=True,blank=True)
    def __str__(self):
        return self.name
    def __unicode__(self):
        return unicode(self.name)
    def _member_list(self):
        if self.members is None:
            return []
        m=self.members.replace(","," ").replace("\n"," ").replace("\r"," ").upper().split()
        m.sort()
        return m
    member_list=property(_member_list)
    def _rpsl(self):
        sep="remark: %s"%("-"*72)
        s=[]
        if self.rpsl_header:
            s+=self.rpsl_header.split("\n")
        s+=["as-set: %s"%self.name]
        for m in self.member_list:
            s+=["members: %s"%m]
        if self.rpsl_footer:
            s+=[sep]
            s+=self.rpsl_footer.split("\n")
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    def rpsl_link(self):
        return "<A HREF='/peer/AS-SET/%s/rpsl/'>RPSL</A>"%self.name
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True

class PeeringPoint(models.Model):
    class Meta:
        verbose_name="Peering Point"
        verbose_name_plural="Peering Points"
    hostname=models.CharField("FQDN",max_length=64,unique=True)
    location=models.CharField("Location",max_length=64,blank=True,null=True)
    local_as=models.ForeignKey(AS,verbose_name="Local AS")
    router_id=models.IPAddressField("Router-ID",unique=True)
    profile_name=models.CharField("Profile",max_length=128,choices=profile_registry.choices)
    communities=models.CharField("Import Communities",max_length=128,blank=True,null=True)
    def __str__(self):
        if self.location:
            return "%s (%s)"%(self.hostname,self.location)
        else:
            return self.hostname
    def __unicode__(self):
        if self.location:
            return u"%s (%s)"%(self.hostname,self.location)
        else:
            return self.hostname
    def sync_cm_prefix_list(self):
        peers_pl=sets.Set([])
        peers_pl.update([p.import_filter_name for p in self.peer_set.filter(import_filter_name__isnull=False) if p.import_filter_name.strip()])
        peers_pl.update([p.export_filter_name for p in self.peer_set.filter(export_filter_name__isnull=False) if p.export_filter_name.strip()])
        h=self.hostname+"/"
        l_h=len(h)
        for p in PrefixList.objects.filter(repo_path__startswith=h):
            pl=p.path[l_h:]
            if pl not in peers_pl:
                p.delete()
            else:
                del peers_pl[pl]
        for pl in peers_pl:
            o=PrefixList(repo_path=h+pl)
            o.save()
    #
    # Returns a list of (prefix-list-name, rpsl-filter)
    #
    def _generated_prefix_lists(self):
        pls={}
        for pr in self.peer_set.all():
            if pr.import_filter_name:
                pls[pr.import_filter_name]=pr.import_filter
            if pr.export_filter_name:
                pls[pr.export_filter_name]=pr.export_filter
        return pls.items()
    generated_prefix_lists=property(_generated_prefix_lists)
    #
    def _profile(self):
        return profile_registry[self.profile_name]()
    profile=property(_profile)
    #
    def _rpsl(self):
        ifaddrs={}
        peers={}
        for p in self.peer_set.all():
            ifaddrs[p.local_ip,p.masklen]=None
            peers[p.remote_ip,p.remote_asn]=None
        s=[]
        s+=["inet-rtr: %s"%self.hostname]
        s+=["local-as: AS%d"%self.local_as.asn]
        for ip,masklen in sorted(ifaddrs.keys(),lambda x,y:cmp(x[0],y[0])):
            s+=["ifaddr: %s masklen %d"%(ip,masklen)]
        for remote_ip,remote_as in sorted(peers.keys(),lambda x,y:cmp(x[0],y[0])):
            s+=["peer: BGP4 %s asno(%s)"%(remote_ip,remote_as)]
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    def rpsl_link(self):
        return "<A HREF='/peer/INET-RTR/%s/rpsl/'>RPSL</A>"%self.hostname
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True

class PeerGroup(models.Model):
    class Meta:
        verbose_name="Peer Group"
        verbose_name_plural="Peer Groups"
    name=models.CharField("Name",max_length=32,unique=True)
    description=models.CharField("Description",max_length=64)
    communities=models.CharField("Import Communities",max_length=128,blank=True,null=True)
    max_prefixes=models.IntegerField("Max. Prefixes",default=100)
    def __str__(self):
        return self.name
    def __unicode__(self):
        return unicode(self.name)
        
class Peer(models.Model):
    class Meta:
        verbose_name="Peer"
        verbose_name_plural="Peers"
    peer_group=models.ForeignKey(PeerGroup,verbose_name="Peer Group")
    peering_point=models.ForeignKey(PeeringPoint,verbose_name="Peering Point")
    local_asn=models.ForeignKey(AS,verbose_name="Local AS")
    local_ip=models.IPAddressField("Local IP")
    masklen=models.PositiveIntegerField("Masklen",default=30)
    remote_asn=models.IntegerField("Remote AS")
    remote_ip=models.IPAddressField("Remote IP")
    import_filter=models.CharField("Import filter",max_length=64)
    local_pref=models.IntegerField("Local Pref",null=True,blank=True)
    export_filter=models.CharField("Export filter",max_length=64)
    description=models.CharField("Description",max_length=64,null=True,blank=True)
    tt=models.IntegerField("TT",blank=True,null=True)
    communities=models.CharField("Import Communities",max_length=128,blank=True,null=True)   # In addition to PeerGroup.communities
                                                                                            # and PeeringPoint.communities
    max_prefixes=models.IntegerField("Max. Prefixes",default=100)
    import_filter_name=models.CharField("Import Filter Name",max_length=64,blank=True,null=True)
    export_filter_name=models.CharField("Export Filter Name",max_length=64,blank=True,null=True)
    def __str__(self):
        return "%s (%s@%s)"%(self.remote_asn,self.remote_ip,self.peering_point.hostname)
    def __unicode__(self):
        return unicode(str(self))
    def save(self):
        if self.import_filter_name is not None and not self.import_filter_name.strip():
            self.import_filter_name=None
        if self.export_filter_name is not None and not self.export_filter_name.strip():
            self.export_filter_name=None
        super(Peer,self).save()
        self.peering_point.sync_cm_prefix_list()
    def _tt_url(self):
        return tt_url(self)
    tt_url=property(_tt_url)
    def admin_tt_url(self):
        return admin_tt_url(self)
    admin_tt_url.short_description="TT"
    admin_tt_url.allow_tags=True
    def admin_import_filter(self):
        r=[]
        if self.import_filter:
            r.append(self.import_filter)
        if self.import_filter_name:
            r.append("(%s)"%self.import_filter_name)
        return "<BR/>".join(r)
    admin_import_filter.short_description="Import Filter"
    admin_import_filter.allow_tags=True
    def admin_export_filter(self):
        r=[]
        if self.export_filter:
            r.append(self.export_filter)
        if self.export_filter_name:
            r.append("(%s)"%self.export_filter_name)
        return "<BR/>".join(r)
    admin_export_filter.short_description="Export Filter"
    admin_export_filter.allow_tags=True
    def _all_communities(self):
        r={}
        for cl in [self.peering_point.communities,self.peer_group.communities,self.communities]:
            if cl is None:
                continue
            for c in cl.replace(","," ").split():
                r[c]=None
        c=r.keys()
        c.sort()
        return " ".join(c)
    all_communities=property(_all_communities)
    def _rpsl(self):
        s="import: from AS%d"%self.remote_asn
        s+=" at %s"%self.peering_point.hostname
        if self.local_pref:
            s+=" action pref=%d;"%self.local_pref
        s+=" accept %s\n"%self.import_filter
        s+="export: to AS%s at %s announce %s"%(self.remote_asn,self.peering_point.hostname,self.export_filter)
        return s
    rpsl=property(_rpsl)
    def _effective_max_prefixes(self):
        if self.max_prefixes:
            return self.max_prefixes
        if self.peer_group.max_prefixes:
            return self.peer_group.max_prefixes
        return 0
    effective_max_prefixes=property(_effective_max_prefixes)
##
## Application Menu
##
class AppMenu(Menu):
    app="peer"
    title="Peering Management"
    items=[
        ("ASes",          "/admin/peer/as/"           ,"peer.change_as"),
        ("ASsets",        "/admin/peer/asset/"        ,"peer.change_asset"),
        ("Communities",   "/admin/peer/community/"    ,"peer.change_community"),
        ("Peering Points","/admin/peer/peeringpoint/" ,"peer.change_peeringpoint"),
        ("Peers",         "/admin/peer/peer/"         ,"peer.change_peer"),
        ("Setup",[
            ("Peer Groups",     "/admin/peer/peergroup/"    , "peer.change_peergroup"),
            ("Community Types", "/admin/peer/communitytype/", "peer.change_communitytype"),
            ("RIRs",            "/admin/peer/rir/",           "peer.change_rir"),
            ("Persons",         "/admin/peer/person/",        "peer.change_person"),
        ])
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM sa_managedobject WHERE name=%s",["ROOT"])[0][0]==0:
            administrative_domain_id=db.execute("SELECT id FROM sa_administrativedomain ORDER BY id")[0][0]
            activator_id=db.execute("SELECT id FROM sa_activator ORDER BY id")[0][0]
            db.execute("""INSERT INTO sa_managedobject(name,is_managed,administrative_domain_id,activator_id,profile_name,scheme,
                        address,is_configuration_managed)
                        VALUES(%s,%s,%s,%s,%s,%s,%s,%s)""",["ROOT",True,administrative_domain_id,activator_id,"NOC",1,"0.0.0.0",False]
            )
    
    def backwards(self):
        "Write your backwards migration here"




from south.db import db
from noc.main.models import *

MIME_TYPES=[
    (".pdf"      , "application/pdf"),
    (".sig"      , "application/pgp-signature"),
    (".spl"      , "application/futuresplash"),
    (".class"    , "application/octet-stream"),
    (".ps"       , "application/postscript"),
    (".torrent"  , "application/x-bittorrent"),
    (".dvi"      , "application/x-dvi"),
    (".gz"       , "application/x-gzip"),
    (".pac"      , "application/x-ns-proxy-autoconfig"),
    (".swf"      , "application/x-shockwave-flash"),
    (".tar.gz"   , "application/x-tgz"),
    (".tgz"      , "application/x-tgz"),
    (".tar"      , "application/x-tar"),
    (".zip"      , "application/zip"),
    (".mp3"      , "audio/mpeg"),
    (".m3u"      , "audio/x-mpegurl"),
    (".wma"      , "audio/x-ms-wma"),
    (".wax"      , "audio/x-ms-wax"),
    (".ogg"      , "application/ogg"),
    (".wav"      , "audio/x-wav"),
    (".gif"      , "image/gif"),
    (".jpg"      , "image/jpeg"),
    (".jpeg"     , "image/jpeg"),
    (".png"      , "image/png"),
    (".xbm"      , "image/x-xbitmap"),
    (".xpm"      , "image/x-xpixmap"),
    (".xwd"      , "image/x-xwindowdump"),
    (".css"      , "text/css"),
    (".html"     , "text/html"),
    (".htm"      , "text/html"),
    (".js"       , "text/javascript"),
    (".asc"      , "text/plain"),
    (".c"        , "text/plain"),
    (".cpp"      , "text/plain"),
    (".log"      , "text/plain"),
    (".conf"     , "text/plain"),
    (".text"     , "text/plain"),
    (".txt"      , "text/plain"),
    (".spec"     , "text/plain"),
    (".py"       , "text/plain"),
    (".dtd"      , "text/xml"),
    (".xml"      , "text/xml"),
    (".mpeg"     , "video/mpeg"),
    (".mpg"      , "video/mpeg"),
    (".mov"      , "video/quicktime"),
    (".qt"       , "video/quicktime"),
    (".avi"      , "video/x-msvideo"),
    (".asf"      , "video/x-ms-asf"),
    (".asx"      , "video/x-ms-asf"),
    (".wmv"      , "video/x-ms-wmv"),
    (".bz2"      , "application/x-bzip"),
    (".tbz"      , "application/x-bzip-compressed-tar"),
    (".tar.bz2"  , "application/x-bzip-compressed-tar"),
    (".odt"      , "application/vnd.oasis.opendocument.text"),
    (".ods"      , "application/vnd.oasis.opendocument.spreadsheet"),
    (".odp"      , "application/vnd.oasis.opendocument.presentation"),
    (".odg"      , "application/vnd.oasis.opendocument.graphics"),
    (".odc"      , "application/vnd.oasis.opendocument.chart"),
    (".odf"      , "application/vnd.oasis.opendocument.formula"),
    (".odi"      , "application/vnd.oasis.opendocument.image"),
    (".odm"      , "application/vnd.oasis.opendocument.text-master"),
    (".ott"      , "application/vnd.oasis.opendocument.text-template"),
    (".ots"      , "application/vnd.oasis.opendocument.spreadsheet-template"),
    (".otp"      , "application/vnd.oasis.opendocument.presentation-template"),
    (".otg"      , "application/vnd.oasis.opendocument.graphics-template"),
    (".otc"      , "application/vnd.oasis.opendocument.chart-template"),
    (".otf"      , "application/vnd.oasis.opendocument.formula-template"),
    (".oti"      , "application/vnd.oasis.opendocument.image-template"),
    (".oth"      , "application/vnd.oasis.opendocument.text-web")
]

class Migration:
    
    def forwards(self):
        for ext,mime_type in MIME_TYPES:
            db.execute("INSERT INTO main_mimetype(extension,mime_type) VALUES(%s,%s)",[ext,mime_type])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Spanning-tree (STP) and variations (MST, RSTP, PVST+, etc) events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## STP Topology changed
##
class STPTopologyChange(EventClass):
    name="STP Topology Change"
    category="NETWORK"
    priority="MINOR"
    subject_template="STP Topology changed: {{interface}}"
    body_template="""STP Topology changed at interface {{interface}}"""
    class Vars:
        interface=Var(required=True)




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.delete_column("fm_eventclassificationrule","drop_event")
        
    def backwards(self):
        db.add_column("fm_eventclassificationrule","drop_event",models.BooleanField("Drop Event",default=False))
        db.execute("UPDATE fm_eventclassificationrule SET drop_event=TRUE WHERE action='D'")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Default Event class
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var

##
## Default event class assigned when no matching event class found
##
class Default(EventClass):
    name="DEFAULT"
##
## DROP
##
class DROP(EventClass):
    name     = "DROP"
    category = "DEFAULT"
    priority = "DEFAULT"
    subject_template="DROPPED EVENT"
    body_template="""DROPPED EVENT"""



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Creole 1.0 Parser (http://www.wikicreole.org)
## Requires python-creole
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import noc.kb.parsers

##
## Macro wrapper
##
class MacroWrapper(object): pass
macro_wrapper=MacroWrapper()
for n,c in noc.kb.parsers.macro_registry.classes.items():
    setattr(macro_wrapper,n,c.expand)
##
## Creole Parser
##
class Parser(noc.kb.parsers.Parser):
    name="Creole"
    @classmethod
    def to_html(cls,kb_entry):
        def custom_link_emit(node):
            if node.children:
                text=html_emitter.emit_children(node)
            else:
                text=None
            return cls.convert_link(kb_entry,node.content,text)
        def custom_image_emit(node):
            target=cls.convert_attach(kb_entry,node.content)
            text=html_emitter.get_text(node)
            return u'<img src="%s" alt="%s" />' % (html_emitter.attr_escape(target),html_emitter.attr_escape(text))
        import creole
        parser=creole.Parser(unicode(kb_entry.body))
        html_emitter=creole.HtmlEmitter(parser.parse(),macros=macro_wrapper)
        html_emitter.link_emit=custom_link_emit
        html_emitter.image_emit=custom_image_emit
        return html_emitter.emit()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## RADIUS events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## RADIUS Alive
##
class RADIUSAlive(EventClass):
    name     = "RADIUS Alive"
    category = "NETWORK"
    priority = "NORMAL"
    subject_template="Radius server responding: {{ip}}"
    body_template="""Radius server responding: {{ip}}"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        ip=Var(required=True,repeat=False)
##
## RADIUS Dead
##
class RADIUSDead(EventClass):
    name     = "RADIUS Dead"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="Radius server not responding: {{ip}}"
    body_template="""Radius server not responding: {{ip}}"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        ip=Var(required=True,repeat=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Add EventClass.is_builtin and EventClassificationRule.is_builtin
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("fm_eventclass","is_builtin",models.BooleanField("Is Builtin",default=False))
        db.add_column("fm_eventclassificationrule","is_builtin",models.BooleanField("Is Builtin",default=False))
    
    def backwards(self):
        db.delete_column("fm_eventclass","is_builtin")
        db.delete_column("fm_eventclassificationrule","is_builtin")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="fm.managed_object_events"
    title="Managed Object Summary"
    requires_cursor=True
    columns=[
        Column("Managed Object"),
        Column("Events",align="RIGHT")]
    
    def get_queryset(self):
        return self.execute("""
            SELECT mo.name,COUNT(*)
            FROM fm_event e JOIN sa_managedobject mo ON (e.managed_object_id=mo.id)
            GROUP BY 1
            ORDER BY 2 DESC""")




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## DLink.DGS3xxx.get_config
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetConfig

class Script(noc.sa.script.Script):
    name="DLink.DGS3xxx.get_config"
    implements=[IGetConfig]
    def execute(self):
        self.cli("disable clipaging")
        v=self.scripts.get_version()
        if "3612" in v["platform"]:
            config=self.cli("show config active")
        else:
            config=self.cli("show config current_config")
        return self.cleaned_config(config)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard models module
## For VC application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from django.db.models import Q
from noc.main.menu import Menu
from noc.main.search import SearchResult
from noc.lib.validators import is_int
##
## Virtual circuit domain, allows to separate unique VC spaces
##
class VCDomain(models.Model):
    class Meta:
        verbose_name="VC Domain"
        verbose_name_plural="VC Domains"
    name=models.CharField("Name",max_length=64,unique=True)
    description=models.TextField("Description",blank=True,null=True)
    
    def __unicode__(self):
        return self.name

##
## Virtual circuit checks
## type -> (name, min labels, l1range, l2range)
## where l1/l2 ranges are tuples of (min,max) or None
##
vc_checks={
    "q": ("802.1Q VLAN",        1, (1,4095),     None),
    "Q": ("802.1ad Q-in-Q",     2, (1,4095),     (1,4095)),
    "D": ("FR DLCI",            1, (16,991),     None),
    "M": ("MPLS",               1, (16,1048575), (16,1048575)),
    "A": ("ATM VCI/VPI",        1, (0,65535),    (0,4095)),
    "X": ("X.25 group/channel", 2, (0,15),       (0,255)),
}
vc_choices=[(k,v[0]) for k,v in vc_checks.items()]
##
## Virtual circuit
##
class VC(models.Model):
    class Meta:
        verbose_name="VC"
        verbose_name_plural="VCs"
        unique_together=[("vc_domain","type","l1","l2")]
    vc_domain=models.ForeignKey(VCDomain,verbose_name="VC Domain")
    type=models.CharField("Type",max_length=1,choices=vc_choices)
    l1=models.IntegerField("Label 1")
    l2=models.IntegerField("Label 2",default=0)
    description=models.CharField("Description",max_length=256)

    def __unicode__(self):
        s=u"%s %s %d"%(self.vc_domain,self.type,self.l1)
        if self.l2:
            s+=u"/%d"%self.l2
        return s
    
    def save(self):
        def in_range(v,r):
            return (v>=r[0]) and (v<=r[1]) 
        description,min_labels,l1range,l2range=vc_checks[self.type]
        if not in_range(self.l1,l1range):
            raise Exception("Invalid value for L1")
        if min_labels>1 and not self.l2:
            raise Exception("L2 required for type %s"%self.type)
        if min_labels>1 and not in_range(self.l2,l2range):
            raise Exception("Invalid value for L2")
        super(VC,self).save()
    ##
    ## Search engine
    ##
    @classmethod
    def search(cls,user,search,limit):
        if user.has_perm("vc.change_vc"):
            if is_int(search):
                tag=int(search)
                for r in VC.objects.filter(Q(l1=tag)|Q(l2=tag))[:limit]:
                    if r.l2:
                        label="%d,%d"%(r.l1,r.l2)
                    else:
                        label="%d"%r.l1
                    yield SearchResult(url="/admin/vc/vc/%d/"%r.id,
                        title="VC: %s, Domain: %s, Label=%s"%(r.type,r.vc_domain.name,label),
                        text=r.description,
                        relevancy=1.0)
##
## Application Menu
##
class AppMenu(Menu):
    app="vc"
    title="Virtual Circuit Management"
    items=[
        ("Virtual Circuits", "/admin/vc/vc/", "vc.change_vc"),
        ("Setup",[
            ("VC Domains", "/admin/vc/vcdomain/", "vc.change_vcdomain"),
        ])
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetDot11Associations
import re

rx_assoc_line=re.compile(r"^(?P<mac>[0-9a-f]{4}\.[0-9a-f]{4}\.[0-9a-f]{4}) (?P<ip>[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}).*$")

class Script(noc.sa.script.Script):
    name="Cisco.IOS.get_dot11_associations"
    implements=[IGetDot11Associations]
    def execute(self):
        self.cli("terminal length 0")
        assoc=self.cli("show dot11 associations")
        r=[]
        for l in assoc.split("\n"):
            match=rx_assoc_line.match(l.strip())
            if match:
                r.append({"mac":match.group("mac"),"ip":match.group("ip")})
        return r




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("fm_eventclassificationrule","action",models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A"))
        db.execute("UPDATE fm_eventclassificationrule SET action='D' WHERE drop_event=TRUE")

    def backwards(self):
        db.delete_column("fm_eventclassificationrule","action")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    depends_on=(
        ("sa","0006_default_activator"),
    )
    def forwards(self):
        a_id=db.execute("SELECT id FROM sa_activator LIMIT 1")[0][0]
        for handler_class_name,repo_path,profile_name,scheme,address,port,user,password,super_password,path\
            in db.execute("SELECT handler_class_name,repo_path,profile_name,scheme,address,port,\"user\",password,super_password,remote_path FROM cm_object"):
            if handler_class_name=="config":
                db.execute("INSERT INTO cm_config(repo_path,activator_id,profile_name,scheme,address,port,\"user\",password,super_password,remote_path) VALUES(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)",
                    [repo_path,a_id,profile_name,scheme,address,port,user,password,super_password,path])
            elif handler_class_name=="dns":
                db.execute("INSERT INTO cm_dns(repo_path) VALUES(%s)",[repo_path])
            elif handler_class_name=="prefix-list":
                db.execute("INSERT INTO cm_prefixlist(repo_path) VALUES(%s)",[repo_path])
            else:
                raise Exception("Unsupported handler_class_name='%s'"%handler_class_name)
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Database File Storage (PostgreSQL only)
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from django.core.files.storage import Storage
from django.core.files.base import File,ContentFile
from django.conf import settings
import psycopg2
##
## Database File Storage (PostgreSQL)
## Stores files in database table.
## Table must have at least following columns:
##   File Name (Character Type) - contains full virtual path
##   Binary Data (BYTEA) - File content
##   Size (Integer Type) - File size (in octets)
##   Modification Time (Timestamp) - Last modification time
## Table name and field names must be set up via
## "option" variable. "option" is a hash with following keys:
##   db_table
##   name_field
##   data_field
##   size field
##   mtime_field
##
class DatabaseStorage(Storage):
    def __init__(self,option):
        self.db_table    = option["db_table"]
        self.name_field  = option["name_field"]
        self.data_field  = option["data_field"]
        self.size_field  = option["size_field"]
        self.mtime_field = option["mtime_field"]
    ##
    ##
    ##
    def get_cursor(self):
        from django.db import connection
        return connection.cursor()
    ##
    ## Internal implementation of the open()
    ##   name - is a full path
    ##   mode - is a file mode (ignored)
    ## Returns File object or None
    ##
    def _open(self, name, mode="rb"):
        cursor=self.get_cursor()
        cursor.execute("SELECT %s FROM %s WHERE %s=%%s"%(self.data_field,self.db_table,self.name_field),[name])
        row=cursor.fetchone()
        if row is None:
            return None
        return ContentFile(row[0])
    ##
    ## Internal implementation of save()
    ##   name - is a full path
    ##   content - is a File object
    ## Returns file name
    ##
    def _save(self, name, content):
        cursor=self.get_cursor()
        binary=content.read()
        size=len(binary)
        binary=psycopg2.Binary(binary)
        if self.exists(name):
            cursor.execute("UPDATE %s SET %s=%%s,%s='now',%s=%%s WHERE %s=%%s"%(self.db_table,self.data_field,
                    self.mtime_field,self.size_field,self.name_field),[binary,size,name])
        else:
            cursor.execute("INSERT INTO %s(%s,%s,%s,%s) VALUES(%%s,'now',%%s,%%s)"%(self.db_table,self.name_field,
                self.mtime_field,self.size_field,self.data_field),[name,size,binary])
        return name
    ##
    ## Check a file exists
    ##   name - is a full path
    ## Returns boolean
    ##
    def exists(self,name):
        cursor=self.get_cursor()
        cursor.execute("SELECT COUNT(*) FROM %s WHERE %s=%%s"%(self.db_table,self.name_field),[name])
        return cursor.fetchone()[0]>0
    ##
    ## Returns full filesystem path as specified in Storage API
    ## Raises NotImplementedError because Storage is not related to filesystem
    ##
    def path(self,name):
        raise NotImplementedError
    ##
    ## Returns file size or 0
    ##
    def size(self,name):
        cursor=self.get_cursor()
        cursor.execute("SELECT %s FROM %s WHERE %s=%%s"%(self.size_field,self.db_table,self.name_field),[name])
        row=cursor.fetchone()
        if row:
            return row[0]
        else:
            return 0
    ##
    ##
    ##
    def url(self,name):
        pass
    ##
    ## Deletes file
    ##    name is a full path
    ##
    def delete(self,name):
        cursor=self.get_cursor()
        cursor.execute("DELETE FROM %s WHERE %s=%%s"%(self.db_table,self.name_field),[name])
    ##
    ## Returns converted file name (Required by Storage API)
    ##
    def get_available_name(self,name):
        return name
    ##
    ## Returns a directory listing
    ##   name - is a full path of directory
    ## Returns empty list when directory does not exists
    ##
    def listdir(self,name):
        cursor=self.get_cursor()
        if not name.endswith("/"):
            name+="/"
        ln=len(name)
        cursor.execute("SELECT %s FROM %s WHERE %s ~ '^%s[^/]+/?$'"%(self.name_field,self.db_table,self.name_field,name.replace("'","\\'")))
        return [x[0][ln:] for x in cursor.fetchall()]
    ##
    ## Returns file stats.
    ## stats is a hash of
    ##    name  : full path
    ##    size  : file size
    ##    mtime : last modification time
    ## Returns None when file does not exists
    def stat(self,name):
        cursor=self.get_cursor()
        cursor.execute("SELECT %s,%s FROM %s WHERE %s=%%s"%(self.size_field,self.mtime_field,self.db_table,self.name_field),[name])
        row=cursor.fetchone()
        if row:
            return {
                "name" : "name",
                "size" : row[0],
                "mtime": row[1],
            }
        else:
            return None
    ##
    ## Set file's mtime
    ##
    def set_mtime(self,name,mtime):
        cursor=self.get_cursor()
        cursor.execute("UPDATE %s SET %s=%%s WHERE %s=%%s"%(self.db_table,self.mtime_field,self.name_field),[mtime,name])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.ip.models import *

class Migration:
    depends_on=(
        ("peer" ,"0001_initial"),
    )
    
    def forwards(self):
        
        # Model 'VRFGroup'
        db.create_table('ip_vrfgroup', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("VRF Group",unique=True,max_length=64)),
            ('unique_addresses', models.BooleanField("Unique addresses in group"))
        ))
        
        # Mock Models
        VRFGroup = db.mock_model(model_name='VRFGroup', db_table='ip_vrfgroup', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'VRF'
        db.create_table('ip_vrf', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("VRF name",unique=True,max_length=64)),
            ('vrf_group', models.ForeignKey(VRFGroup,verbose_name="VRF Group")),
            ('rd', models.CharField("rd",unique=True,max_length=21)),
            ('tt', models.IntegerField("TT",blank=True,null=True))
        ))
        
        # Mock Models
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        VRF = db.mock_model(model_name='VRF', db_table='ip_vrf', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'IPv4BlockAccess'
        db.create_table('ip_ipv4blockaccess', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('user', models.ForeignKey(User,verbose_name=User)),
            ('vrf', models.ForeignKey(VRF,verbose_name=VRF)),
            ('prefix', models.CharField("prefix",max_length=18)),
            ('tt', models.IntegerField("TT",blank=True,null=True))
        ))
        db.create_index('ip_ipv4blockaccess', ['user_id','vrf_id','prefix'], unique=True, db_tablespace='')
        
        
        # Mock Models
        VRF = db.mock_model(model_name='VRF', db_table='ip_vrf', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        AS = db.mock_model(model_name='AS', db_table='peer_as', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'IPv4Block'
        db.create_table('ip_ipv4block', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('description', models.CharField("Description",max_length=64)),
            ('prefix', models.CharField("prefix",max_length=18)),
            ('vrf', models.ForeignKey(VRF)),
            ('asn', models.ForeignKey(AS)),
            ('modified_by', models.ForeignKey(User,verbose_name=User)),
            ('last_modified', models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)),
            ('tt', models.IntegerField("TT",blank=True,null=True))
        ))
        db.create_index('ip_ipv4block', ['prefix','vrf_id'], unique=True, db_tablespace='')
        
        
        # Mock Models
        VRF = db.mock_model(model_name='VRF', db_table='ip_vrf', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'IPv4Address'
        db.create_table('ip_ipv4address', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('vrf', models.ForeignKey(VRF,verbose_name=VRF)),
            ('fqdn', models.CharField("FQDN",max_length=64)),
            ('ip', models.IPAddressField("IP")),
            ('description', models.CharField("Description",blank=True,null=True,max_length=64)),
            ('modified_by', models.ForeignKey(User,verbose_name=User)),
            ('last_modified', models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)),
            ('tt', models.IntegerField("TT",blank=True,null=True))
        ))
        db.create_index('ip_ipv4address', ['vrf_id','ip'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('ip', ['VRFGroup','VRF','IPv4BlockAccess','IPv4Block','IPv4Address'])
    
    def backwards(self):
        db.delete_table('ip_ipv4address')
        db.delete_table('ip_ipv4block')
        db.delete_table('ip_ipv4blockaccess')
        db.delete_table('ip_vrf')
        db.delete_table('ip_vrfgroup')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for KB application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.kb.views import index,index_bookmarks,index_latest,index_popular,index_all,view,attachment,bookmark

urlpatterns = patterns ( None,
        (r"^$",                login_required(index)),
        (r"^bookmarks/$",      login_required(index_bookmarks)),
        (r"^latest/$",         login_required(index_latest)),
        (r"^popular/$",        login_required(index_popular)),
        (r"^all/$",            login_required(index_all)),
        (r"^(?P<kb_id>\d+)/$", login_required(view)),
        (r"^(?P<kb_id>\d+)/attachment/(?P<name>.+)/$", login_required(attachment)),
        (r"^(?P<kb_id>\d+)/bookmark/(?P<action>set|unset)/$", login_required(bookmark)),
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## BINDv9 Zone Generator
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.dns.generators
import time

class Generator(noc.dns.generators.Generator):
    name="BINDv9"
    def get_soa(self):
        nses=["\tNS\t%s\n"%n for n in self.zone.ns_list]
        nses="".join(nses)
        contact=self.zone.profile.zone_contact.replace("@",".")
        if not contact.endswith("."):
            contact+="."
        return """$ORIGIN .
$TTL %(ttl)d
%(domain)s IN SOA %(soa)s %(contact)s (
            %(serial)s ; serial
            %(refresh)d       ; refresh (%(pretty_refresh)s)
            %(retry)d        ; retry (%(pretty_retry)s)
            %(expire)d    ; expire (%(pretty_expire)s)
            %(ttl)d       ; minimum (%(pretty_ttl)s)
            )
%(nses)s
"""%{
            "domain"        : self.zone.name,
            "soa"           : self.zone.profile.zone_soa,
            "contact"       : contact,
            "serial"        : self.zone.serial,
            "ttl"           : self.zone.profile.zone_ttl,
            "pretty_ttl"    : self.pretty_time(self.zone.profile.zone_ttl),
            "refresh"       : self.zone.profile.zone_refresh,
            "pretty_refresh": self.pretty_time(self.zone.profile.zone_refresh),
            "retry"         : self.zone.profile.zone_retry,
            "pretty_retry"  : self.pretty_time(self.zone.profile.zone_retry),
            "expire"        : self.zone.profile.zone_expire,
            "pretty_expire" : self.pretty_time(self.zone.profile.zone_expire),
            "nses"          : nses
        }
        
    def get_records(self):
        s="$ORIGIN %s\n"%self.zone.name
        s+=self.format_3_columns(self.zone.records)
        return s
        
    def get_include(self,ns):
        def ns_list(ds):
            s=[ns.ip for ns in ds.order_by("name") if ns.ip]
            if s:
                return "; ".join(s)+";"
            else:
                return None
        s="""#
# WARNING: This is auto-generated file
# Do not edit manually
#
"""
        # id -> (zone, is_master, masters, slaves)
        zones={}
        for p in ns.masters.all():
            masters=ns_list(p.masters)
            slaves=ns_list(p.slaves)
            for z in p.dnszone_set.filter(is_auto_generated=True):
                zones[z.id]=(z.name,True,masters,slaves)
        for p in ns.slaves.all():
            masters=ns_list(p.masters)
            slaves=ns_list(p.slaves)
            for z in p.dnszone_set.filter(is_auto_generated=True):
                zones[z.id]=(z.name,False,masters,slaves)
        zones=zones.values()
        zones.sort(lambda x,y:cmp(x[0],y[0]))
        for zone,is_master,masters,slaves in zones:
            s+="zone %s {\n"%zone
            if is_master:
                s+="    type master;\n"
                s+="    file \"autozones/%s\";\n"%zone
                if slaves:
                    s+="    allow-transfer {%s};\n"%slaves
            else:
                s+="    type slave;\n"
                s+="    file \"slave/%s\";\n"%zone
                if masters:
                    s+="    masters {%s};\n"%masters
            s+="};\n"
        s+="""#
# End of auto-generated file
#
"""
        return s



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## IP address manipulation routines
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import struct,socket
##
##
##
def address_to_int(ip):
    """
    >>> address_to_int("192.168.0.1")
    3232235521L
    >>> address_to_int("10.0.0.0")
    167772160
    """
    return struct.unpack("!L",socket.inet_aton(ip))[0]
##
##
##
def int_to_address(i):
    """
    >>> int_to_address(3232235521L)
    '192.168.0.1'
    >>> int_to_address(167772160)
    '10.0.0.0'
    """
    return socket.inet_ntoa(struct.pack("!L",i))
##
## Converts bits to an integer
##
def bits_to_int(bits):
    """
    >>> int_to_address(bits_to_int(24))
    '255.255.255.0'
    >>> int_to_address(bits_to_int(16))
    '255.255.0.0'
    """
    return ((1L<<bits)-1L)<<(32L-bits)
##
## Converts bits to netmask
##
def bits_to_netmask(bits):
    """
    >>> bits_to_netmask(24)
    '255.255.255.0'
    >>> bits_to_netmask(16)
    '255.255.0.0'
    """
    try:
        bits=int(bits)
        if bits<=0 or bits>32:
            raise Exception
    except:
        return "255.255.255.255"
    return int_to_address(bits_to_int(bits))

##
## Returns amount of addresses into prefix of "bits" length
##
def bits_to_size(bits):
    return 1L<<(32L-bits)
##
##
##
def prefix_to_size(prefix):
    n,m=prefix.split("/")
    return bits_to_size(int(m))
##
##
##
def network(prefix):
    n,m=prefix.split("/")
    m=int(m)
    return int_to_address(address_to_int(n)&bits_to_int(m))
##
## Convert arbitrary ip/bits pair to strict network/bits
##
def normalize_prefix(prefix):
    n,m=prefix.split("/")
    m=int(m)
    return "%s/%d"%(int_to_address(address_to_int(n)&bits_to_int(m)),m)
##
## Check wrether address is in prefix
##
def contains(prefix,address):
    """
    >>> contains("10.0.0.0/8","192.168.0.1")
    False
    >>> contains("10.0.0.0/8","10.10.12.5")
    True
    >>> contains("10.0.0.0/16","10.10.12.5")
    False
    """
    n,m=prefix.split("/")
    return network(prefix)==network(address+"/%d"%int(m))
##
##
##
def broadcast(prefix):
    n,m=prefix.split("/")
    m=int(m)
    return int_to_address(address_to_int(n)|(0xFFFFFFFFL^bits_to_int(m)))
##
##
##
def wildcard(prefix):
    n,m=prefix.split("/")
    m=int(m)
    return int_to_address(0xFFFFFFFFL^bits_to_int(m))

##
##
##
def prefix_to_bin(prefix):
    """ 
    Convert prefix to a list of bits
    >>> prefix_to_bin("4.0.0.0/8")
    [0, 0, 0, 0, 0, 1, 0, 0]
    >>> prefix_to_bin("192.168.254.19/32")
    [1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1]
    """
    ip,l=prefix.split("/")
    l=int(l)
    i=address_to_int(ip)
    r=[]
    i=i>>(32-l)
    for j in range(l):
        r=[int(i&1)]+r
        i>>=1
    return r
##
##
##
def bin_to_prefix(s):
    """
    Convert list of bits to prefix
    >>> bin_to_prefix([0, 0, 0, 0, 0, 1, 0, 0])
    '4.0.0.0/8'
    >>> bin_to_prefix([1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1])
    '192.168.254.19/32'
    """
    r=0L
    for c in s:
        r=(r<<1)|c
    l=len(s)
    r<<=(32-l)
    return "%s/%d"%(int_to_address(r),l)

##
## Returns a list of free block in "prefix"
## Allocated should be sorted list
##
def free_blocks(prefix,allocated):
    def boundary(p):
        n,m=p.split("/")
        m=int(m)
        a=address_to_int(n)
        b=bits_to_int(m)
        return (a&b,a|(0xFFFFFFFFL^b))
    # Return a minimal list of prefixes coverting f and t addresses
    def cover_blocks(f,t,b=[]):
        def rank(i):
            j=32
            n=1
            while (i&n)==0 and j:
                j-=1
                n=(n<<1)|1
            return j
        n=rank(f)
        while True:
            size=bits_to_size(n)
            if t-f==size-1:
                return b+[int_to_address(f)+"/%d"%n]
            elif n==32 or t-f>size-1:
                return cover_blocks(f+size,t,b+[int_to_address(f)+"/%d"%n])
            n+=1
    if not allocated:
        # No allocations given. Return whole block
        return [prefix]
    used=[]
    p0,p1=boundary(prefix)
    for a in allocated:
        a0,a1=boundary(a)
        if a0<p0 or a1>p1:
            continue
        if used and a0-used[-1]==1:
            # Merge adjanced allocations
            used[-1]=a1
        else:
            used+=[a0,a1]
    free=[]
    while used:
        u0=used.pop(0)
        u1=used.pop(0)
        free+=[u0-1,u1+1]
    if p0<free[0]:
        free.insert(0,p0)
    else:
        free.pop(0)
    if p1>free[-1]:
        free+=[p1]
    else:
        free.pop(-1)
    r=[]
    while free:
        f=free.pop(0)
        t=free.pop(0)
        r+=cover_blocks(f,t)
    return r
##
## Returns minimal prefix containing both IP addresses
##
def minimal_prefix(ip1,ip2):
    """
    >>> minimal_prefix("192.168.0.1","192.168.0.2")
    '192.168.0.0/30'
    >>> minimal_prefix("192.168.0.1","192.168.0.3")
    '192.168.0.0/30'
    >>> minimal_prefix("192.168.0.1","192.168.0.4")
    '192.168.0.0/29'
    >>> minimal_prefix("192.168.0.1","192.168.0.8")
    '192.168.0.0/28'
    >>> minimal_prefix("192.168.0.1","192.168.0.127")
    '192.168.0.0/25'
    >>> minimal_prefix("192.168.0.1","192.168.0.255")
    '192.168.0.0/24'
    >>> minimal_prefix("192.168.0.1","10.0.0.15")
    '0.0.0.0/0'
    >>> minimal_prefix("10.1.12.7","10.0.0.15")
    '10.0.0.0/15'
    """
    for b in range(32,-1,-1):
        suffix="/%d"%b
        n1=network(ip1+suffix)+suffix
        if n1==network(ip2+suffix)+suffix:
            return n1

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from noc.main.report import report_registry
from noc.main.menu import populate_reports
from noc.main.menu import Menu
from noc.lib.fields import BinaryField
from noc.lib.database_storage import DatabaseStorage as DBS
import noc.main.search # Set up signal handlers
import os

##
## Languages
##
class Language(models.Model):
    class Meta:
        verbose_name="Language"
        verbose_name_plural="Languages"
        ordering=("name",)
    name=models.CharField("Name",max_length=32,unique=True)
    native_name=models.CharField("Native Name",max_length=32)
    is_active=models.BooleanField("Is Active",default=False)
    def __unicode__(self):
        return self.name
##
##
##
class DatabaseStorage(models.Model):
    class Meta:
        verbose_name="Database Storage"
        verbose_name_plural="Database Storage"
    name=models.CharField("Name",max_length=256,unique=True)
    data=BinaryField("Data")
    size=models.IntegerField("Size")
    mtime=models.DateTimeField("MTime")
    ##
    ## Options for DatabaseStorage
    ##
    @classmethod
    def dbs_options(cls):
        return {
            "db_table"     : DatabaseStorage._meta.db_table,
            "name_field"  : "name",
            "data_field"  : "data",
            "mtime_field" : "mtime",
            "size_field"  : "size",
        }
    ##
    ## Returns DatabaseStorage instance
    ##
    @classmethod
    def get_dbs(cls):
        return DBS(cls.dbs_options())
##
## Default database storage
##
database_storage=DatabaseStorage.get_dbs()
##
## Extension to MIME-type mapping
##
class MIMEType(models.Model):
    class Meta:
        verbose_name="MIME Type"
        verbose_name_plural="MIME Types"
    extension=models.CharField("Extension",max_length=32,unique=True)
    mime_type=models.CharField("MIME Type",max_length=63)
    def __unicode__(self):
        return u"%s -> %s"%(self.extension,self.mime_type)
    ##
    ## Determine MIME type from filename
    ##
    @classmethod
    def get_mime_type(cls,filename):
        r,ext=os.path.splitext(filename)
        try:
            m=MIMEType.objects.get(extension=ext)
            return m.mime_type
        except MIMEType.DoesNotExist:
            return "application/octet-stream"

##
## Application Menu
##
class AppMenu(Menu):
    app="main"
    title="Main"
    items=[
        ("Setup", [
            ("Users",  "/admin/auth/user/",  "auth.change_user"),
            ("Groups", "/admin/auth/group/", "auth.change_group"),
            ("Languages","/admin/main/language/", "main.change_language"),
            ("MIME Types", "/admin/main/mimetype/", "main.change_mimetype"),
        ]),
        ("Documentation", [
            ("Administrator's Guide", "/static/doc/en/ag/html/index.html"),
            ("User's Guide", "/static/doc/en/ug/html/index.html"),
        ]),
    ]
##
## Load and register reports
##
report_registry.register_all()
populate_reports([(n,r.title) for n,r in report_registry.classes.items()])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Various debugging utilities
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import sys,re,logging,datetime,os,tempfile,cPickle,time,gc

#
# Error reporting context
#
DEBUG_CTX_COMPONENT=None
DEBUG_CTX_CRASH_DIR=None
DEBUG_CTX_CRASH_PREFIX="crashinfo-"
#
def set_crashinfo_context(component,crash_dir):
    global DEBUG_CTX_COMPONENT,DEBUG_CTX_CRASH_DIR
    DEBUG_CTX_COMPONENT=component
    DEBUG_CTX_CRASH_DIR=crash_dir

# Borrowed from django
def get_lines_from_file(filename,lineno,context_lines,loader=None,module_name=None):
    """
    Returns context_lines before and after lineno from file.
    Returns (pre_context_lineno, pre_context, context_line, post_context).
    """
    source = None
    if loader is not None and hasattr(loader, "get_source"):
        source = loader.get_source(module_name)
        if source is not None:
            source = source.splitlines()
    if source is None:
        try:
            f = open(filename)
            try:
                source = f.readlines()
            finally:
                f.close()
        except (OSError, IOError):
            pass
    if source is None:
        return None, [], None, []
    encoding = 'ascii'
    for line in source[:2]:
        # File coding may be specified. Match pattern from PEP-263
        # (http://www.python.org/dev/peps/pep-0263/)
        match = re.search(r'coding[:=]\s*([-\w.]+)', line)
        if match:
            encoding = match.group(1)
            break
    source = [unicode(sline, encoding, 'replace') for sline in source]
    lower_bound = max(0, lineno - context_lines)
    upper_bound = lineno + context_lines
    pre_context = [line.strip('\n') for line in source[lower_bound:lineno]]
    context_line = source[lineno].strip('\n')
    post_context = [line.strip('\n') for line in source[lineno+1:upper_bound]]
    return lower_bound, pre_context, context_line, post_context

# Borrowed from django
def get_traceback_frames(tb):
    frames = []
    while tb is not None:
        # support for __traceback_hide__ which is used by a few libraries
        # to hide internal frames.
        if tb.tb_frame.f_locals.get('__traceback_hide__'):
            tb = tb.tb_next
            continue
        filename = tb.tb_frame.f_code.co_filename
        function = tb.tb_frame.f_code.co_name
        lineno = tb.tb_lineno - 1
        loader = tb.tb_frame.f_globals.get('__loader__')
        module_name = tb.tb_frame.f_globals.get('__name__')
        pre_context_lineno, pre_context, context_line, post_context = get_lines_from_file(filename, lineno, 7, loader, module_name)
        if pre_context_lineno is not None:
            frames.append({
                'tb': tb,
                'filename': filename,
                'function': function,
                'lineno': lineno + 1,
                'vars': tb.tb_frame.f_locals.items(),
                'id': id(tb),
                'pre_context': pre_context,
                'context_line': context_line,
                'post_context': post_context,
                'pre_context_lineno': pre_context_lineno + 1,
            })
        tb = tb.tb_next
    if not frames:
        frames = [{
            'filename': 'unknown',
                    'function': '?',
                    'lineno': '?',
                    'context_line': '???',
                }]
    return frames

def get_execution_frames(frame):
    e_f=[]
    while frame is not None:
        e_f.append(frame)
        frame=frame.f_back
    e_f.reverse()
    frames = []
    for frame in e_f:
        filename = frame.f_code.co_filename
        function = frame.f_code.co_name
        lineno = frame.f_lineno - 1
        loader = frame.f_globals.get('__loader__')
        module_name = frame.f_globals.get('__name__')
        pre_context_lineno, pre_context, context_line, post_context = get_lines_from_file(filename, lineno, 7, loader, module_name)
        if pre_context_lineno is not None:
            frames.append({
                'filename': filename,
                'function': function,
                'lineno': lineno + 1,
                'vars': frame.f_locals.items(),
                'pre_context': pre_context,
                'context_line': context_line,
                'post_context': post_context,
                'pre_context_lineno': pre_context_lineno + 1,
            })
    if not frames:
        frames = [{
            'filename': 'unknown',
                    'function': '?',
                    'lineno': '?',
                    'context_line': '???',
                }]
    return frames

def format_frames(frames):
    def format_source(lineno,lines):
        r=[]
        for l in lines:
            r+=["%5d     %s"%(lineno,l)]
            lineno+=1
        return "\n".join(r)
    r=[]
    r+=["START OF TRACEBACK"]
    r+=["-"*72]
    for f in frames:
        r+=["File: %s (Line: %d)"%(f["filename"],f["lineno"])]
        r+=["Function: %s"%(f["function"])]
        r+=[format_source(f["pre_context_lineno"],f["pre_context"])]
        r+=["%5d ==> %s"%(f["lineno"],f["context_line"])]
        r+=[format_source(f["lineno"]+1,f["post_context"])]
        r+=["Variables:"]
        for n,v in f["vars"]:
            r+=["%20s = %s"%(n,repr(v))]
        r+=["-"*72]
    r+=["END OF TRACEBACK"]
    return "\n".join(r)

def error_report():
    t,v,tb=sys.exc_info()
    now=datetime.datetime.now()
    r=["UNHANDLED EXCEPTION (%s)"%str(now)]
    r+=["Working directory: %s"%os.getcwd()]
    r+=[str(t),str(v)]
    r+=[format_frames(get_traceback_frames(tb))]
    r="\n".join(r)
    logging.error(r)
    if DEBUG_CTX_COMPONENT and DEBUG_CTX_CRASH_DIR:
        c={
            "source"    : "system",
            "type"      : "Unhandled Exception",
            "ts"        : int(time.time()),
            "component" : DEBUG_CTX_COMPONENT,
            "traceback" : r,
        }
        h,p=tempfile.mkstemp(suffix="",prefix=DEBUG_CTX_CRASH_PREFIX,dir=DEBUG_CTX_CRASH_DIR)
        f=os.fdopen(h,"w")
        f.write(cPickle.dumps(c))
        f.close()

def frame_report(frame):
    now=datetime.datetime.now()
    r=["EXECUTION FRAME REPORT (%s)"%str(now)]
    r+=["Working directory: %s"%os.getcwd()]
    r+=[format_frames(get_execution_frames(frame))]
    logging.error("\n".join(r))
##
## Garbage collection profiling
## Instance created on daemon start
##
class GCStats(object):
    def __init__(self):
        self.history={}
    ##
    def get_history(self,t):
        if t in self.history:
            return str(self.history[t])
        else:
            return "[]"
    ##
    ## Collect GC info and prepare report
    ##
    def report(self):
        # Collect all collectable trash
        gc.collect()
        # Calculate type counts
        counts={}
        for obj in gc.get_objects():
            objtype=type(obj)
            if objtype in counts:
                counts[objtype]+=1
            else:
                counts[objtype]=1
        for t,c in counts.iteritems():
            if t in self.history:
                self.history[t]=([c]+self.history[t])[:10]
            else:
                self.history[t]=[c]
        sc=sorted(counts.iteritems(),lambda x,y:-cmp(x[1],y[1]))
        r=["OBJECT REFERENCE COUNT:"]
        r+=["Total objects: %d"%len(gc.get_objects())]
        r+=["%80s: %10d %s"%("%s.%s"%(c[0].__module__,c[0].__name__),c[1],self.get_history(c[0])) for c in sc]
        r+=["END OF OBJECT REFERENCE COUNT"]
        return "\n".join(r)
    ##
    ##
    ##
    def report_dot(self):
        gc.collect()
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.ip.models import *
from noc.lib.fields import CIDRField

class Migration:
    
    def forwards(self):
        db.delete_column("ip_ipv4block","prefix")
        db.add_column("ip_ipv4block","prefix",CIDRField("prefix",null=True))
        db.execute("UPDATE ip_ipv4block SET prefix=prefix_cidr")
        db.delete_column("ip_ipv4block","prefix_cidr")
        db.execute("ALTER TABLE ip_ipv4block ALTER prefix SET NOT NULL")
        db.execute("DROP TRIGGER t_ip_ipv4block_modify ON ip_ipv4block")
        db.execute("DROP FUNCTION f_trigger_ip_ipv4block()")
        db.delete_column("ip_ipv4blockaccess","prefix")
        db.add_column("ip_ipv4blockaccess","prefix",CIDRField("prefix",null=True))
        db.execute("UPDATE ip_ipv4blockaccess SET prefix=prefix_cidr")
        db.delete_column("ip_ipv4blockaccess","prefix_cidr")
        db.execute("ALTER TABLE ip_ipv4blockaccess ALTER prefix SET NOT NULL")
        db.execute("DROP TRIGGER t_ip_ipv4blockaccess_modify ON ip_ipv4blockaccess")
        db.execute("DROP FUNCTION f_trigger_ip_ipv4blockaccess()")
        db.execute(RAW_SQL)
    
    def backwards(self):
        "Write your backwards migration here"

RAW_SQL="""
CREATE OR REPLACE
FUNCTION ip_ipv4_block_depth(INTEGER,CIDR,CIDR)
RETURNS INTEGER
AS $$
DECLARE
    vrf         ALIAS FOR $1;
    inner_block ALIAS FOR $2;
    outer_block ALIAS FOR $3;
    c   INTEGER;
BEGIN
    SELECT COUNT(*)
    INTO   c
    FROM ip_ipv4block
    WHERE vrf_id=vrf AND prefix >> inner_block AND prefix << outer_block;

    RETURN c;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE
FUNCTION ip_ipv4_block_depth_in_vrf_group(INTEGER,CIDR,CIDR)
RETURNS INTEGER
AS $$
DECLARE
    p_vrf_group_id ALIAS FOR $1;
    inner_block    ALIAS FOR $2;
    outer_block    ALIAS FOR $3;
    c   INTEGER;
BEGIN
    SELECT COUNT(*)
    INTO   c
    FROM   ip_ipv4block b JOIN ip_vrf v ON (b.vrf_id=v.id)
    WHERE  v.vrf_group_id=p_vrf_group_id AND prefix >> inner_block AND prefix << outer_block;

    RETURN c;
END;
$$ LANGUAGE plpgsql;
"""



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Link rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,DROP_EVENT
from noc.fm.rules.classes.link import *
from noc.fm.rules.classes.default import DROP
##
## Cisco.IOS Link Up SNMP
##
class Cisco_IOS_Link_Up_SNMP(ClassificationRule):
    name="Cisco.IOS Link Up SNMP"
    event_class=LinkUp
    preference=2000
    required_mibs=["IF-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.6\.3\.1\.1\.5\.4$"), # linkUp
        (r"^1\.3\.6\.1\.2\.1\.2\.2\.1\.2\.\d+$",r"(?P<interface>.*)"),            # ifDescr
    ]
##
## Cisco.IOS Link Up Syslog SNMP
##
class Cisco_IOS_Link_Up_Syslog_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Link Up Syslog SNMP"
    event_class=LinkUp
    preference=1000
    required_mibs=["CISCO-SYSLOG-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"), # clogMessageGenerated
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.\d+$",r"^LINK$"),                 # clogHistFacility
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.\d+$",r"^UPDOWN$"),               # clogHistMsgName
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.5\.\d+$",r"^Interface (?P<interface>.+), changed state to up$"), # clogHistMsgText
    ]
##
## Cisco.IOS Link Up SYSLOG
##
class Cisco_IOS_Link_Up_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS Link Up SYSLOG"
    event_class=LinkUp
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%LINK-3-UPDOWN: Interface (?P<interface>.+), changed state to up$"),
    ]
##
## Cisco.IOS Link Down SNMP
##
class Cisco_IOS_Link_Down_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Link Down SNMP"
    event_class=LinkDown
    preference=2000
    required_mibs=["IF-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.6\.3\.1\.1\.5\.3$"), # linkDown
        (r"^1\.3\.6\.1\.2\.1\.2\.2\.1\.2\.\d+$",r"(?P<interface>.*)"),            # ifDescr
    ]
##
## Cisco.IOS Link Down Syslog SNMP
##
class Cisco_IOS_Link_Down_Syslog_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Link Down Syslog SNMP"
    event_class=LinkDown
    preference=1000
    required_mibs=["CISCO-SYSLOG-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"), # clogMessageGenerated
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.\d+$",r"^LINK$"),                 # clogHistFacility
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.\d+$",r"^UPDOWN$"),               # clogHistMsgName
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.5\.\d+$",r"^Interface (?P<interface>.+), changed state to down$"), # clogHistMsgText
    ]
##
## Cisco.IOS Link Down SYSLOG
##
class Cisco_IOS_Link_Down_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS Link Down SYSLOG"
    event_class=LinkDown
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%LINK-3-UPDOWN: Interface (?P<interface>.+), changed state to down$"),
    ]
##
## Cisco.IOS Line Protocol SYSLOG
##
class Cisco_IOS_Line_Protocol_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS Line Protocol SYSLOG"
    event_class=DROP
    preference=1000
    action=DROP_EVENT
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%LINEPROTO-5-UPDOWN"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM pg_language WHERE lanname='plpgsql'")[0][0]==0:
            db.execute("CREATE LANGUAGE plpgsql")
        db.execute(CREATE_F)
        db.execute(CREATE_T)
    
    def backwards(self):
        db.execute(DROP_T)
        db.execute(DROP_F)

CREATE_F="""
CREATE OR REPLACE
FUNCTION f_sa_task_insert()
RETURNS TRIGGER
AS $$
BEGIN
    NOTIFY sa_new_task;
    RETURN NULL;
END;
$$ LANGUAGE plpgsql;
"""

CREATE_T="""
CREATE TRIGGER t_sa_task_insert
AFTER INSERT ON sa_task
FOR EACH STATEMENT EXECUTE PROCEDURE f_sa_task_insert();
"""

DROP_T="""
DROP TRIGGER IF EXISTS t_sa_task_insert ON sa_task;
"""

DROP_F="""
DROP FUNCTION f_sa_task_insert();
"""



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'MIB'
        db.create_table('fm_mib', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('description', models.TextField("Description",blank=True,null=True)),
            ('last_updated', models.DateTimeField("Last Updated")),
            ('uploaded', models.DateTimeField("Uploaded"))
        ))
        
        # Mock Models
        MIB = db.mock_model(model_name='MIB', db_table='fm_mib', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'MIBData'
        db.create_table('fm_mibdata', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('mib', models.ForeignKey(MIB,verbose_name=MIB)),
            ('oid', models.CharField("OID",max_length=128,unique=True)),
            ('name', models.CharField("Name",max_length=128,unique=True)),
            ('description', models.TextField("Description",blank=True,null=True))
        ))
        
        db.send_create_signal('fm', ['MIB','MIBData'])
    
    def backwards(self):
        db.delete_table('fm_mibdata')
        db.delete_table('fm_mib')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Mercurial support
## (http://www.selenic.com/mercurial/wiki/)
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.cm.vcs
import os,datetime

class VCS(noc.cm.vcs.VCS):
    name="hg"
    def check_repository(self):
        if not os.path.exists(os.path.join(self.repo,".hg")):
            self.cmd(["init"],check=False)
    def log(self,path):
        revs=[]
        for l in self.cmd_out(["log","--template","{rev} {date}\n",path]).split("\n"):
            l=l.strip()
            if not l:
                continue
            rev,date=l.split(" ")
            d,r=date.split("-",1)
            revs+=[noc.cm.vcs.Revision(rev,datetime.datetime.fromtimestamp(float(d)))]
        return revs
    def diff(self,path,rev1,rev2):
        return self.cmd_out(["diff","-r%s:%s"%(rev1.revision,rev2.revision),path])
    def get_revision(self,path,rev):
        return self.cmd_out(["cat","-r%s"%rev.revision,path])
    def mv(self,f,t):
        self.cmd(["mv",f,t])
        self.commit(f,"mv")
        self.commit(t,"mv")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="sa.config_by_group"
    title="Configs by Groups"
    requires_cursor=True
    columns=[Column("Group"),Column("Qty")]
    
    def get_queryset(self):
        return self.execute("""
            SELECT g.name,COUNT(*)
            FROM sa_managedobject o
                JOIN sa_managedobject_groups og ON (o.id=og.managedobject_id)
                JOIN sa_objectgroup g ON (og.objectgroup_id=g.id)
            GROUP BY 1 ORDER BY 2 DESC""")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

COLORS={
    "CRITICAL":("#BB0000","#FFDDCC"),
    "MAJOR"   :("#BB0000","#FFEEDD"),
    "MINOR"   :("#BB0000","#FFFFBB"),
    "WARNING" :("#BB0000","#FFFFDD"),
    "NORMAL"  :("#BB0000","#A8C2C2"),
    "INFO"    :("#BB0000","#DCF3F3"),
    "DEFAULT" :("#FFFF00","#643200"),
}

class Migration:
    
    def forwards(self):
        for p,colors in COLORS.items():
            font,bg=colors
            r=db.execute("SELECT id,font_color,background_color FROM fm_eventpriority WHERE name=%s",[p])
            if len(r)==1:
                pid,dbf,dbg=r[0]
                if not dbf:
                    db.execute("UPDATE fm_eventpriority SET font_color=%s WHERE id=%s",[font,pid])
                if not dbf:
                    db.execute("UPDATE fm_eventpriority SET background_color=%s WHERE id=%s",[bg,pid])
                
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    depends_on=(
        ("sa","0005_activator"),
    )
    def forwards(self):
        Activator = db.mock_model(model_name='Activator', db_table='sa_activator', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Config'
        db.create_table('cm_config', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('repo_path', models.CharField("Repo Path",max_length=128,unique=True)),
            ('push_every', models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)),
            ('next_push', models.DateTimeField("Next Push",blank=True,null=True)),
            ('last_push', models.DateTimeField("Last Push",blank=True,null=True)),
            ('pull_every', models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)),
            ('next_pull', models.DateTimeField("Next Pull",blank=True,null=True)),
            ('last_pull', models.DateTimeField("Last Pull",blank=True,null=True)),
            ('activator',models.ForeignKey(Activator,verbose_name="Activator")),
            ('profile_name', models.CharField("Profile",max_length=128,choices=profile_registry.choices)),
            ('scheme', models.IntegerField("Scheme",choices=[(0,"telnet"),(1,"ssh")])),
            ('address', models.CharField("Address",max_length=64)),
            ('port', models.PositiveIntegerField("Port",blank=True,null=True)),
            ('user', models.CharField("User",max_length=32,blank=True,null=True)),
            ('password', models.CharField("Password",max_length=32,blank=True,null=True)),
            ('super_password', models.CharField("Super Password",max_length=32,blank=True,null=True)),
            ('remote_path', models.CharField("Path",max_length=32,blank=True,null=True))
        ))
        # Mock Models
        Config = db.mock_model(model_name='Config', db_table='cm_config', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'Config.categories'
        db.create_table('cm_config_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('config', models.ForeignKey(Config, null=False)),
            ('objectcategory', models.ForeignKey(ObjectCategory, null=False))
        )) 
        # Model 'PrefixList'
        db.create_table('cm_prefixlist', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('repo_path', models.CharField("Repo Path",max_length=128,unique=True)),
            ('push_every', models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)),
            ('next_push', models.DateTimeField("Next Push",blank=True,null=True)),
            ('last_push', models.DateTimeField("Last Push",blank=True,null=True)),
            ('pull_every', models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)),
            ('next_pull', models.DateTimeField("Next Pull",blank=True,null=True)),
            ('last_pull', models.DateTimeField("Last Pull",blank=True,null=True))
        ))
        # Mock Models
        PrefixList = db.mock_model(model_name='PrefixList', db_table='cm_prefixlist', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'PrefixList.categories'
        db.create_table('cm_prefixlist_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('prefixlist', models.ForeignKey(PrefixList, null=False)),
            ('objectcategory', models.ForeignKey(ObjectCategory, null=False))
        )) 
        # Model 'DNS'
        db.create_table('cm_dns', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('repo_path', models.CharField("Repo Path",max_length=128,unique=True)),
            ('push_every', models.PositiveIntegerField("Push Every (secs)",default=86400,blank=True,null=True)),
            ('next_push', models.DateTimeField("Next Push",blank=True,null=True)),
            ('last_push', models.DateTimeField("Last Push",blank=True,null=True)),
            ('pull_every', models.PositiveIntegerField("Pull Every (secs)",default=86400,blank=True,null=True)),
            ('next_pull', models.DateTimeField("Next Pull",blank=True,null=True)),
            ('last_pull', models.DateTimeField("Last Pull",blank=True,null=True))
        ))
        # Mock Models
        DNS = db.mock_model(model_name='DNS', db_table='cm_dns', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'DNS.categories'
        db.create_table('cm_dns_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('dns', models.ForeignKey(DNS, null=False)),
            ('objectcategory', models.ForeignKey(ObjectCategory, null=False))
        )) 
        
        db.send_create_signal('cm', ['Config','PrefixList','DNS'])
    
    def backwards(self):
        db.delete_table('cm_config_categories')
        db.delete_table('cm_prefixlist_categories')
        db.delete_table('cm_dns_categories')
        
        db.delete_table('cm_dns')
        db.delete_table('cm_prefixlist')
        db.delete_table('cm_config')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## SA module views.py
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.shortcuts import get_object_or_404
from django import forms
from noc.lib.render import render, render_success, render_failure
from noc.sa.models import ManagedObject,script_registry,profile_registry,AdministrativeDomain,Activator,scheme_choices
from django.http import HttpResponseForbidden,HttpResponseNotFound,HttpResponseRedirect
from xmlrpclib import ServerProxy, Error
from noc.settings import config
import pprint,types,socket,csv

##
## Display a list of object's scripts
##
def object_scripts(request,object_id):
    o=get_object_or_404(ManagedObject,id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    p=o.profile_name
    scripts=sorted([(p+"."+x,x) for x in profile_registry[p].scripts.keys()])
    return render(request,"sa/scripts.html",{"object":o,"scripts":scripts})
##
## Execute object script
##
def object_script(request,object_id,script):
    def get_result(script,object_id,**kwargs):
        server=ServerProxy("http://%s:%d"%(config.get("xmlrpc","server"),config.getint("xmlrpc","port")))
        try:
            result=server.script(script,object_id,kwargs)
        except socket.error,why:
            raise Exception("XML-RPC socket error: "+why[1])
        if type(result) not in [types.StringType,types.UnicodeType]:
            result=pprint.pformat(result)
        return result
        
    o=get_object_or_404(ManagedObject,id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    try:
        scr=script_registry[script]
    except:
        return HttpResponseNotFound("No script found")
    form=None
    result=None
    if scr.implements and scr.implements[0].requires_input():
        if request.POST:
            form=scr.implements[0].get_form(request.POST)
            if form.is_valid():
                data={}
                for k,v in form.cleaned_data.items():
                    if v:
                        data[k]=v
                try:
                    result=get_result(script,object_id,**data)
                except Exception,why:
                    return render_failure(request,"Script Failed",why)
        else:
            form=scr.implements[0].get_form()
    else:
        try:
            result=get_result(script,object_id)
        except Exception,why:
            return render_failure(request,"Script Failed",why.faultString)
    return render(request,"sa/script.html",{"object":o,"result":result,"script":script,"form":form})
##
## Form for uploading managed objects
##
class MOUploadForm(forms.Form):
    administrative_domain=forms.ModelChoiceField(queryset=AdministrativeDomain.objects)
    activator=forms.ModelChoiceField(queryset=Activator.objects)
    file=forms.FileField()
##
## Managed objects tools
##
def tools(request):
    if not request.user.is_superuser:
        return HttpResponseForbidden("Access Denied")
    return render(request,"sa/tools.html",{"upload_mo_form":MOUploadForm()})
##
## List of CSV file fields
##
MO_IMPORT_CSV_FIELDS=["name","profile_name","description","scheme","address","port","user","password","super_password","remote_path",
    "trap_source_ip","trap_community","snmp_ro","snmp_rw","repo_path"]
MO_IMPORT_REQUIRED_CSV_FIELDS=["name","profile_name","scheme","address","repo_path"]
SCHEME=dict([(x[1],x[0]) for x in scheme_choices])
##
## Upload managed objects from a CSV file
##
def upload_managed_objects(request):
    def upload_csv(file):
        count=0
        reader=csv.reader(file)
        rows=list(reader)
        if len(rows)<=1:
            return 0
        # First row is a field names
        header=rows.pop(0)
        # Check field names
        for c in header:
            if c not in MO_IMPORT_CSV_FIELDS:
                return 0,"Invalid field '%s'"%c
        # Check all required fields present
        for c in MO_IMPORT_REQUIRED_CSV_FIELDS:
            if c not in header:
                return 0,"Required fields '%s' is missed"%c
        # Check rows
        for row in rows:
            vars=dict(zip(header,row)) # Convert row to hash: field->value
            if not vars: # Skip empty lines
                continue 
            try:
                o=ManagedObject.objects.get(name=vars["name"]) # Find existing object
            except ManagedObject.DoesNotExist:
                o=ManagedObject(name=vars["name"])# Or create new
            for k,v in vars.items():
                if k=="name": # "name" is already processed
                    continue
                if k=="profile_name" and v not in profile_registry.classes:
                    return 0,"Invalid profile '%s' at line %d"%(v,count+1)
                if not v: # Skip empty fields
                    continue
                if k=="port": # Convert port to integer
                    try:
                        v=int(v)
                    except:
                        return 0,"Invalid port: %s at line %d"%(v,count+1)
                if k=="scheme":
                    try:
                        v=SCHEME[v]
                    except KeyError:
                        return 0,"Invalid access scheme %s at line %d"%(v,count+1)
                if k=="repo_path": # Set is_configuration_managed if repo_path given
                    o.is_configuration_managed=True
                setattr(o,k,v) # Finally set attribute
            o.administrative_domain=administrative_domain
            o.activator=activator
            o.save() # Perform SQL statement
            count+=1
        return count,None
    # Process request
    if not request.user.is_superuser:
        return HttpResponseForbidden("Access Denied")
    if request.method=="POST":
        form = MOUploadForm(request.POST, request.FILES)
        if form.is_valid():
            administrative_domain=form.cleaned_data["administrative_domain"]
            activator=form.cleaned_data["activator"]
            count,error=upload_csv(request.FILES['file'])
            if error:
                return render_failure(request,"Managed Object Upload Failure",error)
            else:
                return render_success(request,"Managed Objects are Uploaded","%d managed objects uploaded/updated"%count)
    return HttpResponseRedirect("/sa/tools/")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.ip.models import VRF
from lib.validators import is_cidr
from lib.ip import free_blocks,bits_to_size,prefix_to_size
from django import forms

class ReportForm(forms.Form):
    vrf=forms.ModelChoiceField(label="VRF",queryset=VRF.objects)
    prefix=forms.CharField(label="Prefix",initial="0.0.0.0/0")
    
    def clean_prefix(self):
        prefix=self.cleaned_data.get("prefix","").strip()
        if not is_cidr(prefix):
            raise forms.ValidationError("Invalid prefix")
        return prefix

class Report(noc.main.report.Report):
    name="ip.summary"
    title="IP Block summary"
    form_class=ReportForm
    requires_cursor=True
    columns=[Column("Name"),Column("Value",align="RIGHT")]
    
    def get_queryset(self):
        vrf_id=self.form.cleaned_data["vrf"].id
        prefix=self.form.cleaned_data["prefix"]
        allocated=self.execute("""
            SELECT prefix
            FROM ip_ipv4block b
            WHERE vrf_id=%s
                AND prefix<<%s::cidr
                AND (SELECT COUNT(*) FROM ip_ipv4block bb WHERE vrf_id=%s AND bb.prefix<<b.prefix)=0
            ORDER BY prefix
        """,[vrf_id,prefix,vrf_id])
        allocated=[a[0] for a in allocated]
        allocated_30=self.execute("""
            SELECT prefix
            FROM ip_ipv4block b
            WHERE vrf_id=%s
                AND prefix<<%s::cidr
                AND masklen(prefix)=30
                AND (SELECT COUNT(*) FROM ip_ipv4block bb WHERE vrf_id=%s AND bb.prefix<<b.prefix)=0
            ORDER BY prefix
        """,[vrf_id,prefix,vrf_id])
        allocated_30=[a[0] for a in allocated_30]
        free=free_blocks(prefix,allocated)
        m,n=prefix.split("/")
        total_addresses=bits_to_size(int(n))
        allocated_addresses=sum([prefix_to_size(x) for x in allocated])
        allocated_30_addresses=sum([prefix_to_size(x) for x in allocated_30])
        free_addresses=sum([prefix_to_size(x) for x in free])
        a_s=len(allocated)
        if a_s:
            avg_allocated_size=float(allocated_addresses)/float(a_s)
        else:
            avg_allocated_size=0.0
        return [
            ["Total addresses",     total_addresses],
            ["Allocated",           allocated_addresses],
            ["....in /30 networks", allocated_30_addresses],
            ["Free",                free_addresses],
            ["Avegage allocated block size", "%8.2f"%avg_allocated_size]
        ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVlans
import re

rx_vlan_line=re.compile(r"^(?P<vlan_id>\d{1,4})\s+(?P<name>\S+)\s")

class Script(noc.sa.script.Script):
    name="Cisco.IOS.get_vlans"
    implements=[IGetVlans]
    def execute(self):
        self.cli("terminal length 0")
        vlans=self.cli("show vlan brief")
        r=[]
        for l in vlans.split("\n"):
            match=rx_vlan_line.match(l.strip())
            if match:
                name=match.group("name")
                if name in ["fddi-default","trcrf-default","fddinet-default","trbrf-default"]:
                    continue
                r.append({"vlan_id": int(match.group("vlan_id")),
                          "name"   : name})
        return r




from south.db import db
from noc.main.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'MIMEType'
        db.create_table('main_mimetype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('extension', models.CharField("Extension",max_length=32,unique=True)),
            ('mime_type', models.CharField("MIME Type",max_length=63))
        ))
        
        db.send_create_signal('main', ['MIMEType'])
    
    def backwards(self):
        db.delete_table('main_mimetype')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## DLink.DGS3xxx.get_vlans
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVlans
import re

rx_vlan=re.compile(r"VID\s+:\s+(?P<vlanid>\S+).+VLAN Name\s+:\s+(?P<vlanname>\S+)",re.MULTILINE|re.DOTALL)

class Script(noc.sa.script.Script):
    name="DLink.DGS3xxx.get_vlans"
    implements=[IGetVlans]
    def execute(self):
        self.cli("disable clipaging")
        vlans=self.cli("show vlan")
        r=[]
        for l in vlans.split("\n"):
            l=l.strip()
            match=rx_vlan.search(l)
            if match:
                r.append({"vlan_id":int(match.group('vlanid')),"name":match.group('vlanname')})
        return r




from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        db.rename_column("sa_managedobject","location","description")
        
    def backwards(self):
        db.rename_column("sa_managedobject","description","location")




from south.db import db
from noc.kb.models import *

class Migration:
    
    def forwards(self):
        
        
        # Mock Models
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBEntryAttachment'
        db.create_table('kb_kbentryattachment', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('kb_entry', models.ForeignKey(KBEntry,verbose_name="KB Entry")),
            ('name', models.CharField("Name",max_length=256)),
            ('description', models.CharField("Description",max_length=256,null=True,blank=True)),
            ('is_hidden', models.BooleanField("Is Hidden",default=False)),
            ('file', models.FileField("File"))
        ))
        db.create_index('kb_kbentryattachment', ['kb_entry_id','name'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('kb', ['KBEntryAttachment'])
    
    def backwards(self):
        db.delete_table('kb_kbentryattachment')
        




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.execute("DROP FUNCTION update_event_classification(INTEGER,INTEGER,INTEGER,INTEGER,INTEGER,TEXT,TEXT,TEXT[][])")
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Abstract module loader/registry
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import os,logging
##
## Abstract module loader/registry
##
class Registry(object):
    name="Registry"
    subdir="directory"
    classname="Class"
    apps=None
    def __init__(self):
        self.classes={}
        self.choices=[]
        self.is_registered=False
    
    #
    # Should be called within metaclass' __new__ method
    #
    def register(self,name,module):
        if name is None:
            return
        if name in self.classes:
            raise Exception,"Module %s registred twice"%name
        logging.info("%s: Register %s"%(self.name,name))
        self.classes[name]=module
        self.choices.append((name,name))
    #
    # Should be called at the top of the models.py
    #
    def register_all(self):
        if self.is_registered:
            return
        if self.apps is None:
            from django.conf import settings
            apps=[a for a in settings.INSTALLED_APPS if a.startswith("noc.")]
        else:
            apps=self.apps
        for app in apps:
            pd=os.path.join(app[4:],self.subdir)
            if not os.path.isdir(pd):
                continue
            for dirpath,dirnames,filenames in os.walk(pd):
                mb=app+"."+".".join(dirpath.split(os.sep)[1:])+"."
                for f in [f for f in filenames if f.endswith(".py") and f!="__init__.py"]:
                    __import__(mb+f[:-3],{},{},self.classname)
        self.is_registered=True
    #
    #
    #
    def __getitem__(self,name):
        return self.classes[name]




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'EventCorrelationRule'
        db.create_table('fm_eventcorrelationrule', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('rule', models.TextField("Rule")),
            ('description', models.TextField("Description",null=True,blank=True)),
            ('is_builtin', models.BooleanField("Is Builtin",default=False))
        ))
        
        db.send_create_signal('fm', ['EventCorrelationRule'])
    
    def backwards(self):
        db.delete_table('fm_eventcorrelationrule')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard admin module
## For VC application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from noc.vc.models import VCDomain,VC

##
## Admin form for VCDomain
##
class VCDomainAdmin(admin.ModelAdmin):
    list_display=["name"]
    search_fields=["name"]

class VCAdmin(admin.ModelAdmin):
    list_display=["vc_domain","type","l1","l2","description"]
    search_fields=["l1","l2","description"]
    list_filter=["vc_domain","type"]

admin.site.register(VCDomain,VCDomainAdmin)
admin.site.register(VC,VCAdmin)


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Small DSL for event classification rules description
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import re
from noc.fm.rules.classes.default import Default
from noc.fm.models import MIB,EventClass,EventClassificationRule,EventClassificationRE,MIBNotFoundException

re_var=re.compile(r"\(\?P<([^>]+>)\)")
##
##
##
event_class_cache={}
def get_event_class(ec):
    global event_class_cache
    if ec not in event_class_cache:
        event_class_cache[ec.name]=EventClass.objects.get(name=ec.name)
    return event_class_cache[ec.name]

##
## ClassigicationRule metaclass
## Performs regexp patterns checking
##
class ClassificationRuleBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        vars={}
        # Check regular expressions in patters
        for p in m.patterns:
            if p.__class__==Expression:
                pass
            else:
                l,r=p
                try:
                    re.compile(l)
                    re.compile(r,re.MULTILINE|re.DOTALL)
                except:
                    raise Exception("Invalid pattern ('%s','%s') in rule %s"%(l,r,m.name))
        # Check variables
        return m
##
## Rule actions
##
MAKE_ACTIVE="A"
DROP_EVENT="D"
CLOSE_EVENT="C"
##
## Event classification rule
##
class ClassificationRule(object):
    __metaclass__=ClassificationRuleBase
    name="Classification Rule"
    event_class=Default
    preference=1000
    action=MAKE_ACTIVE
    required_mibs=[] # A list of required MIB Names
    patterns=[]  # Pair of (left_re,right_re)
    ##
    ## Syncronize class with database
    ##
    @classmethod
    def sync(cls):
        # Check all required MIBs are uploaded
        for mib_name in cls.required_mibs:
            try:
                mib=MIB.objects.get(name=mib_name)
            except MIB.DoesNotExist:
                raise MIBNotFoundException(mib_name)
        # Create or update event classification rule
        try:
            r=EventClassificationRule.objects.get(name=cls.name)
            r.event_class=get_event_class(cls.event_class)
            r.preference=cls.preference
            r.action=cls.action
            r.is_builtin=True
            print "UPDATE RULE %s"%cls.name
        except EventClassificationRule.DoesNotExist:
            r=EventClassificationRule(
                event_class=get_event_class(cls.event_class),
                name=cls.name,
                preference=cls.preference,
                action=cls.action,
                is_builtin=True
            )
            print "CREATE RULE %s"%cls.name
        r.save()
        r.eventclassificationre_set.all().delete()
        for p in cls.patterns:
            if p.__class__==Expression:
                EventClassificationRE(rule=r,left_re=p.left,right_re=p.right,is_expression=True).save()
            else:
                rx_l,rx_r=p
                EventClassificationRE(rule=r,left_re=rx_l,right_re=rx_r).save()
##
## Wrapper for expression variables
##
class Expression(object):
    def __init__(self,left,right):
        self.left=left
        compile(right,"inline","eval") # Raise SyntaxError in case of invalid expression
        self.right=right



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.core.management.base import BaseCommand
from xmlrpclib import ServerProxy, Error
import ConfigParser

defaults_config_path="etc/%(daemon_name)s.defaults"

class Command(BaseCommand):
    help="Debug SAE XML-RPC server"
    def handle(self, *args, **options):
        config=ConfigParser.SafeConfigParser()
        for p in ["etc/noc-sae.defaults","etc/noc-sae.conf"]:
            config.read(p)
        server=ServerProxy("http://%s:%d"%(config.get("xmlrpc","listen"),config.getint("xmlrpc","port")))
        proxy=getattr(server,args[0])
        print proxy(*args[1:])
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Telephony events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Authentication Failed
##
class PhoneCall(EventClass):
    name="Phone Call"
    category="PHONE"
    priority="INFO"
    subject_template="Phone call: {{calling_number}} -> {{called_number}} (Duration: {{duration}}s, Clearing: {{call_clearing}})"
    body_template="""Phone call:
Calling number: {{calling_number}}
Called number: {{called_number}}
Duration: {{duration}}
Connect time: {{connect_time}}
Disconnect time: {{disconnect_time}}
Call clearing: {{call_clearing}}
"""
    class Vars:
        calling_number=Var(required=True)
        called_number=Var(required=True)
        connect_time=Var(required=True)
        disconnect_time=Var(required=True)
        call_clearing=Var(required=True)
        duration=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Usage: debug-script <profile> <script> <stream-url>
##
## WARNING!!!
## This module implements part of activator functionality.
## Sometimes via dirty hacks
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.core.management.base import BaseCommand
from noc.sa.profiles import profile_registry
from noc.sa.script import script_registry,scheme_id
from noc.sa.activator import Service
from noc.sa.protocols.sae_pb2 import *
from noc.sa.rpc import TransactionFactory
import logging,sys,ConfigParser,Queue,time
from noc.lib.url import URL
from noc.lib.nbsocket import SocketFactory
from optparse import OptionParser, make_option

class Controller(object): pass

##
## Activator emulation
##
class ActivatorStub(object):
    def __init__(self):
        # Simple config stub
        self.config=ConfigParser.SafeConfigParser()
        self.config.add_section("activator")
        self.config.set("activator","max_pull_config","2")
        self.script_call_queue=Queue.Queue()
        self.ping_check_results=None
    
    def tick(self):
        while not self.script_call_queue.empty():
            try:
                f,args,kwargs=self.script_call_queue.get_nowait()
            except:
                break
            logging.debug("Calling delayed %s(*%s,**%s)"%(f,args,kwargs))
            apply(f,args,kwargs)
        
    def on_script_exit(self,script):
        pass
        
    def run_script(self,name,access_profile,callback,**kwargs):
        pv,pos,sn=name.split(".",2)
        profile=profile_registry["%s.%s"%(pv,pos)]()
        script=script_registry[name](profile,self,access_profile,**kwargs)
        script.start()
    
    def request_call(self,f,*args,**kwargs):
        logging.debug("Requesting call: %s(*%s,**%s)"%(f,args,kwargs))
        self.script_call_queue.put((f,args,kwargs))

class Command(BaseCommand):
    help="Debug SA Script"
    option_list=BaseCommand.option_list+(
        make_option("-c","--read-community",dest="snmp_ro"),
    )
    def handle(self, *args, **options):
        def handle_callback(controller,response=None,error=None):
            if error:
                logging.debug("Error: %s"%error.text)
            if response:
                logging.debug("Config pulled")
                logging.debug(response.config)
        if len(args) not in [2,3]:
            print "Usage: debug-script <script> <stream url> [key1=value1,....,keyN=valueN]"
            return
        script_name=args[0]
        vendor,os_name,rest=script_name.split(".",2)
        profile_name="%s.%s"%(vendor,os_name)
        try:
            profile=profile_registry[profile_name]()
        except:
            print "Invalid profile. Available profiles are:"
            print "\n".join([x[0] for x in profile_registry.choices])
            return
        try:
            script_class=script_registry[script_name]
        except:
            print "Invalid script. Available scripts are:"
            print "\n".join([x[0] for x in script_registry.choices])
            return
        logging.root.setLevel(logging.DEBUG)
        service=Service()
        service.activator=ActivatorStub()
        service.activator.factory=SocketFactory(tick_callback=service.activator.tick)
        url=URL(args[1])
        r=ScriptRequest()
        r.script=script_name
        r.access_profile.profile        = profile_name
        r.access_profile.scheme         = scheme_id[url.scheme]
        r.access_profile.address        = url.host
        if url.port:
            r.access_profile.port       = url.port
        r.access_profile.user           = url.user
        if "\x00" in url.password: # Check the password really the pair of password/enable password
            p,s=url.password.split("\x00",1)
            r.access_profile.password   = p
            r.access_profile.super_password = s
        else:
            r.access_profile.password   = url.password
        r.access_profile.path           = url.path
        if options["snmp_ro"]:
            r.access_profile.snmp_ro=options["snmp_ro"]
        # Parse script args
        if len(args)==3:
            for p in args[2].split(","):
                k,v=p.strip().split("=")
                a=r.kwargs.add()
                a.key=k
                a.value=v
        #
        controller=Controller()
        tf=TransactionFactory()
        controller.transaction=tf.begin()
        service.script(controller=controller,request=r,done=handle_callback)
        service.activator.factory.run()
        service.activator.factory.tick_callback()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IPing
import re

rx_result=re.compile(r"^Success rate is \d+ percent \((?P<success>\d+)/(?P<count>\d+)\)(, round-trip min/avg/max = (?P<min>\d+)/(?P<avg>\d+)/(?P<max>\d+) ms)?",re.MULTILINE|re.DOTALL)

class Script(noc.sa.script.Script):
    name="Cisco.IOS.ping"
    implements=[IPing]
    def execute(self,address,count=None,source_address=None,size=None,df=None):
        cmd="ping ip %s"%address
        if count:
            cmd+=" count %d"%int(count)
        if source_address:
            cmd+=" source %s"%source_address
        if size:
            cmd+=" size %d"%int(size)
        if df:
            cmd+=" df-bit"
        pr=self.cli(cmd)
        match=rx_result.search(pr)
        return {
                "success": match.group("success"),
                "count"  : match.group("count"),
                "min"    : match.group("min"),
                "avg"    : match.group("avg"),
                "max"    : match.group("max"),
            }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Application menu builder
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import types
##
## Registered menu list
##
MENU=[]
##
## Populate MENU with reports
##
def populate_reports(reports):
    apps={}
    for r,title in reports:
        app,rest=r.split(".",1)
        if app not in apps:
            apps[app]=[]
        apps[app].append((title,"/main/report/"+r+"/"))
    for m in MENU:
        if m["app"] in apps:
            m["items"].append(("Reports",{"items":apps[m["app"]]}))
##
## Menu metaclass
## Performs menu registration
##
class MenuBase(type):
    def __new__(cls,name,bases,attrs):
        def convert(x):
            if type(x[1])==types.ListType:
                return (x[0],{"items":x[1]})
            else:
                return x
        m=type.__new__(cls,name,bases,attrs)
        if m.app and m.title:
            r={
                "app"   : m.app,
                "title" : m.title,
                "items" : [convert(x) for x in m.items],
            }
            MENU.append(r)
        return m
##
## Application menu class.
##
class Menu(object):
    __metaclass__=MenuBase
    app=None     # Application name
    title=None # Application title
    items=[]   # list of (title,link)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'RIR'
        db.create_table('peer_rir', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("name",max_length=64,unique=True)),
            ('whois', models.CharField("whois",max_length=64,blank=True,null=True))
        ))
        
        # Mock Models
        RIR = db.mock_model(model_name='RIR', db_table='peer_rir', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Person'
        db.create_table('peer_person', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('nic_hdl', models.CharField("nic-hdl",max_length=64,unique=True)),
            ('person', models.CharField("person",max_length=128)),
            ('address', models.TextField("address")),
            ('phone', models.TextField("phone")),
            ('fax_no', models.TextField("fax-no",blank=True,null=True)),
            ('email', models.TextField("email")),
            ('rir', models.ForeignKey(RIR,verbose_name=RIR)),
            ('extra', models.TextField("extra",blank=True,null=True))
        ))
        
        # Mock Models
        RIR = db.mock_model(model_name='RIR', db_table='peer_rir', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Maintainer'
        db.create_table('peer_maintainer', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('maintainer', models.CharField("mntner",max_length=64,unique=True)),
            ('description', models.CharField("description",max_length=64)),
            ('auth', models.TextField("auth")),
            ('rir', models.ForeignKey(RIR,verbose_name=RIR)),
            ('extra', models.TextField("extra",blank=True,null=True))
        ))
        # Mock Models
        Maintainer = db.mock_model(model_name='Maintainer', db_table='peer_maintainer', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        Person = db.mock_model(model_name='Person', db_table='peer_person', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'Maintainer.admins'
        db.create_table('peer_maintainer_admins', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('maintainer', models.ForeignKey(Maintainer, null=False)),
            ('person', models.ForeignKey(Person, null=False))
        )) 
        
        db.send_create_signal('peer', ['RIR','Person','Maintainer'])
        for rir,whois in [
                ("ARIN","whois.arin.net"),
                ("RIPE NCC","whois.ripe.net"),
                ("APNIC","whois.apnic.net"),
                ("LACNIC","whois.lacnic.net"),
                ("AfriNIC","whois.afrinic.net")]:
            db.execute("INSERT INTO peer_rir(name,whois) VALUES(%s,%s)",[rir,whois])
    
    def backwards(self):
        db.delete_table('peer_maintainer_admins')
        db.delete_table('peer_maintainer')
        db.delete_table('peer_person')
        db.delete_table('peer_rir')
